C 
C Copyright:
C C.Capdevila Montes and Dr.C. Garcia de Andres, CENIM, Madrid
C Dr. H. K. D. H. Bhadeshia, University of Cambridge
C C.Capdevila Montes and Dr.C. Garcia de Andres, CENIM, Madrid
C Department of Materials Science and Metallurgy, Pembroke St.
C Cambridge CB2 3QZ, England. Telephone Cambridge 334301.
C
C
C	***********/////////////////////////**************
C
C    GROWTH KINETICS OF PROEUTECTOID FERRITE. 1st APROXIMATION
C
C	***********/////////////////////////**************
C
C Bibligraphy:
C ------------
C	
C -  J.B Gilmour-G.R. Purdy- J.S. Kirkaldy, Met. Trans,
C    Vol. 3, (1972), pp. 3213. 
C -  D.E. Coates, Met. TRans., Vol 4, (1973), pp. 395.
C -  H.K.D.H. Bhadeshia, Progress in Materials Science, 
C    Vol. 29, (1985), pp. 321.
C -  H.I. Aaronson, M.R. Plitchta, G.W. Franti, and K.C. Russell,
C    Met. Trans., Vol. 9A, (1978), pp.363.
C -  M. Takahashi, PhD., University of Cambridge, 1992.
C -  H.K.D.H. Bhadeshia, Metal Science, Vol. 16, March 1982,pp.159.
C -  D.E. Porter & K.E. Easterling,'Phase Transformation in Metals and 
C    Alloys',(1981),Wokingham,(England),Van Nostrand Reinhold.
C -  O.Grong,' Metallurgical Modelling of Welding',(1995),Institute 
C    of Materials,(England),H.K.D.H. Bhadeshia.
C -  L.Kaufman and S.V. Radcliffe,'Decomposition of austenite by diffusional 
C    process',ed. by V.F. Zackay and H.I. Aaronson, Intercience Publishers,
C    New York, NY,(1962),pp.313.
C -  K.C. Russell, Acta Metall.,1969,Vol. 17,pp.1123.
C
C Notes:
C ------
C
C *15/07/97:
C The following calculations deal with the one-dimensional growth
C of planar grain boundary allotriomorphs from opposite side of grain.
C This process may be considered in two stages. The first involves 
C parabolic growth from both sides according to the earlier exact 
C calculations.
C During the second stage of impingment the driving force due to the 
C carbon activity gradient is gradually reduced and the transformation 
C rate correspondingly decrease to a value consistent with a uniform and
C equal carbon chemical potential in both phases. 
C
C Equilibrium concentration at interface is fixed at temperature tested
C
C One-dimensional growth model.
C
C Routine to determine parabolic rate constant using 4th order
C polynomial, fitted to calculated values of DLOG(GES) for OMEGA from 0
C to 1.
C To calculate parabolic rate constant ALPHA3 for 3-D growth
C
C *21/07/97:
C Half-thickness under impingment calculated using RTSAFE
C function. PC's Fortran Power Station 4.0. 
C Using a combination of Newton-Raphson and bisection,
C find the root of a function bracketed between X1 and
C X2. The root returned as the function RTSAFE. 
C
C Program stops calculations when the activity of carbon reachs uniform
C activity, Z=Z2
C
C *24/07/97:
c The concentration of carbon in the middle of the austneite grain size
C is calculated by means of balancing the amount of carbon enrichment
C of the austenite against the carbon depletion of the ferrite 
C (assuming XAGA=0)
C
C The distribution of carbon in front of a ferrite allotriomorph of 
C half-thickness Z in the absence of soft-impingement is given by a linear
C profile (H.K.D.H. Bhadeshia approximation)
C
C The distribution of carbon in front of ferrite under impingement is 
C consider like a straight line.(Gilmour et al, 1972).
C
C *28/07/97:
c Volume fraction of allotriomorphic ferrite is included. Due to symmetry
C of the problem, a standard relationship between half-thickness and 
C austenite grain size is considered like volume fraction.
C
C TIME is in seconds
C VOl is volume fraction of ferrite/equilibrium vol. frac.
C VF is the actual volume fraction
C ALP is the one-dimensional parabolic thickening rate constant
C ALP1 is the one-dimensional parabolic lengthening rate constant
C BI is the grain boundary face nucleation rate per unit area per unit t
C XGAG, XAGA = equilibrium mol frac of carbon in gamma and alpha respect
C XBAR = average mol frac of carbon in alloy
C SV = austenite grain boundary area per unit volume (1/m)
C
C Program to calculate the growth rate of pearlite assuming the bulk 
C or interface diffusion control. Based on 
C p9ferr.f. Copyright, Dr. H. K. D. H. Bhadeshia, University of Cambridge
C Department of Materials Science and Metallurgy, Pembroke St.
C Cambridge CB2 3QZ, England. Telephone Cambridge 334301.
C Version 1.6 (1995)
C
C The method applies to steels containing just one substitutional solute
C but all solutes if paraequilibrium
C 
C Gamma/Gamma+Alpha and Gamma/Gamma+Theta interface composition are 
C calculated using the Newton-Raphson method assuming Hillert-Staffanson
C regular solution model.(1991.5.12 M.TAKAHASHI)
C
C Initial guess values are calculated by Kirkaldy's approximate method
C   J. S. Kirkaldy, B. A. Thomson and E. A. Baganis, 
C   "Hardenability Concepts with Applications to Steel"
C   Editors D. V. Doane and J. S. Kirkaldy
C   Published by The Metallurgical Society of the American Inst. 
C   Mining, Metallurgical and Petroleum Engineers, 
C   Warrendale, Pennsylvania, 1978, pp. 83-125.
C
C Overall transformation kinetics for pearlite
C H K D H Bhadeshia, University of Cambridge (p9ferr.f & pearl2.f)
C 
C The pearlite growth rate in m/s assuming paraequilibrium growth
C with growth controlled by the diffusion of carbon in the austenite.
C
C The pearlite formation is assuming paraequilibrium growth. 
C Pearlite appears when the carbon concentration on the remaining 
c austenite reachs the value of the interface austenite / austenite + theta
C extrapoled at the temperature of the test.
C The rate of austenite / ferrite interface is assumed very slow. 
C VEL1 is the pearlite growth rate in m/s assuming paraequilibrium growth
C  with growth controlled by the diffusion of carbon in the austenite.
C VPEARL is the volume fraction of pearlite in a fully austenitic sample
C SUM is the number of ferrite grains per unit area of austenite grain 
C   surface at the point of site saturation
C N fix the number of nucleation sites per unit volume, for pearlite, based
C  on the number of ferrite grains per unit volume
C ATIME2 calculate the time required to achieve VP of pearlite at current
C temperature. This follows the Scheil procedure.
C BTIME calculate new time appropriate  for current temperature
C Unnormalize VPEARL2 to allow for ferrite
C
C *26/08/97:
C The Pearlite appears when the two conditions propposed by Enomoto are
C achieved:
C	-Carbon concentration into austenite rise the value marked for the 
C	austenite / austenite + theta extrapolated curve at temperature.
C	-The Pearlites becomes the nucleation forming cementite nucleus, for 
C	this reason the speed of the interface should not be over cementite 
C	nucleation.
C The velocity of austenite / ferrite interface is incorporated (VELFER). 
C Based on Guilmour et al (Met. Trans. ,1972).
C 
C Therefore, the restriction that the migration of the alpha / gamma boundary, 
C at which nucleation takes place must not exceed that which displaces this 
C boundary a distance of one lattice parameter, a, in the time required for 
C an embryo to develop to the critical nucleus size should be satisfied.
C (Aaronson, et al, Met. Trans., vol 9A, 1978, pp.363.)
C
C *2/09/97:
C Routine  AE3TEM finds an approximate temperature at which paraequilibrium 
C ferrite formation first becomes possible. Based on 
C (H. K. D. H. Bhadeshia,6 July 1994).
C Routine AE1TEM gives the Pearlite temperature added according
C to:
C		A.K Sinha.'Ferreous Physical Metallurgy'. Ed:Butterworths
C			  USA,1989. pp.202.
C C.Capdevila-Montes. (Cambridge,1997).
C All temeperatures in centigrades.
C
C*11/09/97:
C Calculation of the moving rate of alpha / gamma interface under parabolic
C growth. Based on:
C
C P.R Krahe,K.R.Kinsan and H.I.Aaronson,Acta metall.,vol.20,1972,pp.1109.
C
C*12/09/97:
C Correction of Guilmour paper regarding ecuation [5].:
C ...In ecuation [5], c11(=xfec) is c'11(=xgag1) really 
C NUCLEATION TIME:
C Calculation of nucleation time based on MUCG46.F. Version 1.7 July 1993.
C Copyright Dr. H. K. D. H. Bhadeshia, University of Cambridge
C MUCG46.FOR
C Department of Materials Science and Metallurgy, Pembroke St. Cambridge CB2 3QZ
C
C*15/09/97:
C Change of the interface energy for nucleation of cementite into austenite 
C according the Zener work.     
C 
      IMPLICIT NONE
      DOUBLE PRECISION C(12),WT(12),COMPOS(11),WEIGTH(11)
     &, lineal,W,XGAG1,XAGA1,XAGA,XGAG,DIFFC
     &, CARBON,SI,MN,NI,MO,CR,V,CO,CU,AL,WO,ctemp,ti,XEQ
     &, TIMESTEP,TIMEPROF,SIGMA,DGAMMA,SV,SITE,ZPROF
     &, VELFER,VELMAX,DIFF,NUCTIME
     &, PI,HH,KK,W1,R,T,A5,XBAR,XWT,TFEA,TFEB
	DOUBLE PRECISION XNORM1,Z1,Z2,Z20,GMAX,CON2,TIMIMP
     &, X,ZTOT,ZTOT1,ZTOT2,ZS,CL1,VFIN,TIMSTART,Y,ZRT
     &, DALPHA,TDELTA,ZDELTA,VF,SUM,ZSTAG,TZERO,DELTA,TIM
     &, T1,GV,TIMEFIN,PXGC,PX1GC,X1GC,ZVOL,B
     &, ZP,ZPMAX,ZPEARL,VOLPEAR,VPEARL,VTOT,Y1,ZRT1,CL2
      DOUBLE PRECISION RTSAFE,FIMP,C2,FLO,XALPH,T10,T20
     &, PROFILA,PROFILB,PROFILD,X2AG,PX1AG,PX2AG,S,VEL2
     &, X1GA,X2GA,X1CG,X2CG,X2GC,PX1GA,PX1CG,PX2CG
     &, VEL1,PX2GA,CL,PX2GC,OMEGA,ALPHA,ALPHA1,D,Z
     &, WFE,XFEC,XFEC1,TAE1,TAE3,SC,X1AG,X1
	INTEGER I,IJ,IIP,IIS,IPARA,JP,JPEARL
C
C Natural constants to work
C
      PARAMETER (PI=3.141592654D+00,R=8.31432D+00)
      Z=12.0D+00
	A5=1.0D+00
      HH=6.6262D-34
      KK=1.38062D-23
      W1=48570.0D+00
C
C Reading data regarding chemical composition, austenite grain size,
C temperature and holding time for growing ferrite. Austenite grain 
C in cm.
C
C      READ (5,102) CARBON,SI,MN,NI,MO,CR,V,CO,CU,AL,WO
      READ (5,102) CARBON
	READ (5,102) SI 
     	READ (5,102) MN
     	READ (5,102) NI
     	READ (5,102) MO
	READ (5,102) CR
	READ (5,102) V
	READ (5,102) CO
	READ (5,102) CU
	READ (5,102) AL
	READ (5,102) WO
C     
      READ (5,*) LINEAL
      READ(5,*)CTEMP
      READ(5,*)TI
      READ(5,*)TIMEPROF
102   FORMAT (11f6.4)
114   FORMAT(F6.2)
C
C Initial composition.
C
      T=CTEMP+273.0D+00
153   C(1)=CARBON
      C(2)=SI
      C(3)=MN
      C(4)=NI
      C(5)=MO
      C(6)=CR
      C(7)=V
      C(8)=CO
      C(9)=CU
      C(10)=AL
      C(11)=WO
      DO 2221 I=1,11
	WEIGTH(I)=C(I)
2221  CONTINUE
      WRITE(*,104)(C(I),I=1,7)
      WRITE(*,121)
      WRITE(*,105)CTEMP
      WRITE(*,120)TI
      WRITE(*,115)LINEAL
      LINEAL=LINEAL*1.0D-04
C
C The most suitable TIMESTEP is calculated as follows
C
      CALL STEP(LINEAL,TIMESTEP)
      WRITE(*,2150)TIMESTEP
C
C Concentration of carbon in the interface austenite-ferrite in mole
C fraction (XGAG=XEQ), and the concentration of carbon into ferrite
C (XAGA).
C Concentration in weight percentage (XGAG1 into austenite and XAGA1
C into ferrite.
C
148   CALL NEWOMEGA(C,W,XBAR,T10,T20,0)
      DO 2222 I=1,11
          COMPOS(I)=C(I)
2222  CONTINUE
      WRITE(*,106)W
      WRITE(*,118)W1
      XAGA=XALPH(CTEMP)
      WRITE(*,1001)
      WRITE(*,107)XBAR
      WRITE(*,108)XAGA
      XWT=XAGA
      CALL CONCEN(CTEMP,T10,T20,W,XEQ)
      XGAG=XEQ
      WRITE(*,109)XGAG
      CALL INTERF(CARBON,XGAG,XAGA,C,XGAG1,XAGA1)
C
C Fe-C interface composition
C
      TFEA=0.0D+00
      TFEB=0.0D+00
      WFE=8054.0D+00
      CALL CONCEN(CTEMP,TFEA,TFEB,WFE,XEQ)
      XFEC1=XEQ
      XFEC=XFEC1*(CARBON/XBAR)
C
C Critical temperatures for diffusional decomposition of the austenite
C into ferrite + pearlite.
C Equilibrium temperatures in centigrades.
C
      CALL AE3TEM(COMPOS,W,T10,T20,TAE3)
      CALL AE1TEM(WEIGTH,W,T10,T20,TAE1)
      WRITE(*,2190)
      WRITE(*,2160)TAE3
      WRITE(*,2170)TAE1
      WRITE(*,2200)
C
C Supersaturation
C
      OMEGA=((XGAG-XBAR)/(XGAG-XAGA))
C
C Coefficient of Diffusion of Carbon into austenite with the
C temperature. This coefficient takes account the influence of another
C alloying elements.
C
      T=CTEMP+273.0D+00
      CALL DIFFUS(DIFFC,XBAR,T,W)
      WRITE(*,1010)DIFFC
C
C Parabolic-thickening rate constant for one dimensional growth(ALPHA).
C
      CALL ALPH(W,XBAR,CTEMP,XAGA,XGAG,ALPHA,D)
      CALL ALP(XGAG,XBAR,XAGA,D,CTEMP,ALPHA1)
      WRITE(*,111)ALPHA
      WRITE(*,1434)ALPHA1
      DIFF=D
C
C Calculation of the half-thickness of the layer of ferrite.
C Parabolic growth is considered.
C
      ALPHA=ALPHA1
      Z=ALPHA*SQRT(TI)
      write(*,113)Z
C
C Impingement began when time achieved T1 or the half-thickness achieve
C Z1 value. (Gilmour et al, H.K.D.B Bhadeshia, Progress in materials
c Science, vol. 29, (1985), pp.321.)
C The study consider only half-diameter grain.
C
      LINEAL=0.5*LINEAL 
      XNORM1=CARBON
      Z20=LINEAL*(1-(XNORM1/XFEC))
      Z2=LINEAL*(1-(XNORM1/XGAG1))
      Z1=LINEAL*((XGAG1-XNORM1)/(XGAG1+XNORM1))    
C
C Correction of Guilmour paper regarding the value of C11:
C ...In equation [5], c11(=xfec) is c'11(=xgag1) really 
C
      XFEC=XGAG1
C
      T1=(Z1/ALPHA)**2
C
C # # # # # # # # # # # ----NUCLEATION TIME----# # # # # # # # # # # # # # #
C
C Calculation of nucleation time of allotriomorphic ferrite. 
C Copyright Dr. H. K. D. H. Bhadeshia, University of Cambridge
C Based on MUCG46.FOR. Version 1.7 July 1993.
C Department of Materials Science and Metallurgy, Pembroke St. 
C Cambridge CB2 3QZ
C
      SITE=1.0D-09
      DGAMMA=LINEAL*2.0D-02
      SV=2.0D+00/DGAMMA
      SIGMA=0.05D+00
      CALL AG(C,W,T10,T20,T,GMAX)         
      GV=GMAX/7.2D-06
      T1=(TAE3+273.14)-T
      IF (T1.GT.0.0D+00) THEN
      	NUCTIME=DEXP((0.6031D+06/(8.31432*T))-0.1905D+03+20.0*DLOG(T)
     &	-4*DLOG(DABS(GMAX)))
      ELSE
	WRITE(*,1152)
	GOTO 1745
      END IF
C # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
C
      WRITE(*,1208)Z1
      WRITE(*,1209)Z20
      WRITE(*,6010)z2
      WRITE(*,1800)NUCTIME
      WRITE(*,1201)
      WRITE(*,22)
      WRITE(*,23)
C
C Incoherent input data
C
      IF (TI.LE.NUCTIME) THEN
	 WRITE(*,1743)
	 STOP
      END IF
C
C Study of Gilmour et al function to calculate the time to achieve one
C particular half-thickness, in order to know what is the time which 
C impingement beguin.
C
      CON2=FLO(Z1,LINEAL,C2)
      TIMIMP=FIMP(0.0d+00,xnorm1,xgag1,xfec,lineal,DIFF,CON2)
      TZERO=TIMIMP 
C
C Calculations of half-thickness and concentration in the middle of the 
C austenite grain size (CL - value).
C 
      DELTA=TIMESTEP
      WRITE(*,1205)
      IJ=0
      IIP=0
      DO 1203 TIM=0.0D+00,TI,DELTA
 	  X=TIM-NUCTIME
	  IF (X.LE.0.0D+00) THEN
		ZTOT2=0.0
		ZS=0.0
		CL1=XNORM1
		VFIN=0.0
		VELFER=0.0
		GOTO 1719
          ELSE
          	ZS=ALPHA*SQRT(X)
          	Z=ALPHA*SQRT(X)
         	IF (Z.GE.Z1) IJ=IJ+1
          	IF (Z.GE.Z1.AND.IJ.EQ.1) THEN
			WRITE(*,1202)
			TIMSTART=X
          	END IF
          	IF (Z .LE. Z1) THEN
               		ZTOT=Z
          	ELSE
               		Y=X+TIMIMP
               		ZRT=RTSAFE(XNORM1,XGAG1,xfec,LINEAL,Y,DIFF)
               		ZSTAG=ALPHA*SQRT(X)
               		ZTOT1=ZTOT+ZRT
	       		IIP=IIP+1
	       		IF (IIP.EQ.1) THEN
				ZDELTA=ZRT
				TDELTA=X
	       		END IF
               		ZTOT1=ZTOT1-ZDELTA
               		IF (ZTOT1.GE.Z2) THEN
		   		TIMEFIN=X
                  		WRITE(*,1207)
                   		GOTO 1111
               		END IF
          	END IF
	  END IF
 	  CL1=CL(XNORM1,XGAG1,LINEAL,ZTOT1)
	  IF (CL1.LE.XNORM1) THEN
		CL1=XNORM1
	  END IF
          IF(ZS.LE.Z1) THEN
               ZTOT2=ZTOT
          ELSE
               ZTOT2=ZTOT1
          END IF  
C
C Calculation of volume fraction of allotriomorphic ferrite
C 
	  VF=2.0D-02*ZTOT2/DGAMMA 
	  VFIN=VF
C
C Calculation of the velocity of alpha / gamma interface in (m/s)
C
	  IF (ZS.GT.Z1) THEN
	 	 VELFER=2.0D-02*DIFF*((Z2-ZTOT2)/((LINEAL-ZTOT2)**2))
          ELSE		 
		 IF (X.EQ.0.0D+00) THEN
			VELFER=0.0D+00
		 ELSE
		 	VELFER=0.5D-02*ALPHA/SQRT(X)
		 END IF
          END IF
C
1719      WRITE(*,1206)TIM,ZTOT2,zs,cl1,vfin,velfer
1203  CONTINUE 
      TIMEFIN=X
      TIMEFIN=NUCTIME+X
C
C Advices about non-real data about carbon profile ahead the interface
C
1111  IF (TIMEPROF.GE.TIMEFIN) THEN
		WRITE(*,2100)
      		WRITE(*,2130)
		WRITE(*,2120)
		TIMEPROF=TIMEFIN
		WRITE(*,2100)
		WRITE(*,2110)TIMEPROF
      END IF
      IF (TIMEPROF.LE.NUCTIME) THEN
		WRITE(*,1743)
		GOTO 1745
      END IF
C 
C Calculation of the distribution of carbon in front of a ferrite 
C allotriomorphs in the absence of impingement. A straight line
C is considered.
C
C Under impingement, a straight line is considered (Gilmour, 1972)
C Variables:
C 		Profile-> Distribution of carbon in front of 
C			  allotriomorph (x,time)
C
c Volume fraction of allotriomorphic ferrite is included. Due to symmetry
C of the problem, a standard relationship between half-thickness and 
C austenite grain size is considered like volume fraction. 
C
C The considerations about pearlite are based on the dissertation 
C submitted for the degree of Doctor of Philosophy at the University
C of Cambridge by Manabu Takahashi: 
C
C		'Reaustenitisation from bainite in Steels'
C			      (Chapter 5)
C		University ofCambridge, September 1992
C
C The pearlite formation is assuming paraequilibrium growth. 
C Pearlite appears when the carbon concentration on the remaining 
c austenite reachs the value of the interface austenite / austenite + theta
C extrapoled at the temperature of the test.
C The rate of austenite / ferrite interface is assumed very slow. 
C VEL1 is the pearlite growth rate in m/s assuming paraequilibrium growth
C  with growth controlled by the diffusion of carbon in the austenite.
C VPEARL is the volume fraction of pearlite in a fully austenitic sample
C SUM is the number of ferrite grains per unit area of austenite grain 
C   surface at the point of site saturation
C N fix the number of nucleation sites per unit volume, for pearlite, based
C  on the number of ferrite grains per unit volume
C ATIME2 calculate the time required to achieve VP of pearlite at current
C temperature. This follows the Scheil procedure.
C BTIME calculate new time appropriate  for current temperature
C Unnormalize VPEARL2 to allow for ferrite
C
      WRITE(*,1210)
      WRITE(*,1211)
      WRITE(*,1210)
      SUM=0.2810D+11
      WRITE(*,300)SUM
      DALPHA=(2.0D+00/(3.0D+00*SV*SUM))**0.333333
      DGAMMA=2.0D+00/(SV)
      ipara=1
      jpearl=1
      CALL PEARL(CARBON,SI,MN,NI,MO,CR,0.0D+00,CTEMP
     &           ,1.0d+00,IPARA,
     &           PX1GC,PX2GC,PX1CG,PX2CG, PX1GA,PX2GA,PX1AG,PX2AG, VEL1,
     &           X1GC,X2GC,X1CG,X2CG, X1GA,X2GA,X1AG,X2AG,
     &           VEL2,SC,S,JPEARL )
      C(1)=PX1GC
      PXGC=PX1GC
      CALL CONV(0,WT,C)
      X1GC=WT(1)
      WRITE(*,8)DALPHA,DGAMMA
      WRITE(*,105)CTEMP
      WRITE(*,1212)TIMEPROF  
      WRITE(*,1218)X1GC
      TIMEPROF=TIMEPROF-NUCTIME
      ZPROF=ALPHA*SQRT(TIMEPROF)
C
C Cementite nucleation growth.
C Calculation of the highest growth rate of a disorder alpha / gamma boundary 
C at which a crystal of cementite can nucleate
C
	  CALL CRITICAL(CTEMP,TAE1,PXGC,DIFF,COMPOS,WEIGTH,VELMAX)
	  IF (VELMAX.LE.0.0D+00) VELMAX=0.0D+00
C     
C
C No impingement has taken place
C
      IF (ZPROF.LT.Z1) THEN
          ZVOL=ZPROF*1.0D-02
          VF=2.00D+00*(ZVOL/DGAMMA)
	  VELFER=0.5d-02*ALPHA/SQRT(TIMEPROF)
C
C 		Condition for nucleation of pearlite:
C 		*********** VELFER<VELMAX ***********
C
          WRITE(*,1213)ZPROF 
          WRITE(*,2022)VELFER 
          WRITE(*,1020)VELMAX 
          WRITE(*,1217)VF
          write(*,1214)
          WRITE(*,1216)
	  WRITE(*,2020)
          IIS=0
          DO 1 B=ZPROF,LINEAL,0.1D-03
             X1=PROFILA(B,XNORM1,XGAG1,XAGA1,ZPROF)
             IF (X1.LT.X1GC) IIS=IIS+1
	     IF (IIS.EQ.1) ZP=B
             ZPMAX=ABS(ZP-ZPROF)
             ZPEARL=ABS(B-ZPROF)
             IF (ZPEARL.LE.ZPMAX) THEN
                    VOLPEAR=ZPEARL/LINEAL
             ELSE
                    VOLPEAR=ZPMAX/LINEAL
             END IF
C         
C Allow for prior transformation to allotriomorphic ferrite
C
444          VPEARL=VOLPEAR
	     IF (VELFER.GT.VELMAX) VPEARL=0.0
             VTOT=VF+VPEARL
             write(*,1215)B,x1,VPEARL,VTOT
1         CONTINUE 
      END IF 
C
C Impingement is taken place
C               
      IF (ZPROF.GE.Z1.AND.ZPROF.LT.Z2) THEN 
          Y1=TDELTA+TZERO
          Y=TZERO+TIMEPROF
	  ZRT1=RTSAFE(XNORM1,XGAG1,XFEC,LINEAL,Y1,DIFF)
          ZRT=RTSAFE(XNORM1,XGAG1,xfec,LINEAL,Y,DIFF)
          ZPROF=Z1+ZRT-ZRT1
          ZVOL=ZPROF*1.0D-02
          VF=2.0D+00*(ZVOL/DGAMMA)
	  VELFER=2.0D-02*DIFF*((Z2-ZPROF)/((LINEAL-ZPROF)**2))
          WRITE(*,1213)ZPROF 
          WRITE(*,2022)VELFER 
          WRITE(*,1020)VELMAX 
          WRITE(*,1217)VF
          WRITE(*,1214)
          WRITE(*,1216)
	  WRITE(*,2021)
	  CL2=CL(XNORM1,XGAG1,LINEAL,ZPROF)
          JP=0
          DO 2 B=ZPROF,LINEAL,0.1D-03
             X1=PROFILB(B,CL2,XGAG1,LINEAL,ZPROF)
             IF (CL2.GE.X1GC) THEN
                ZP=LINEAL
                GOTO 555
             END IF
             IF (X1.LT.X1GC) JP=JP+1
	     IF (JP.EQ.1) ZP=B
555          ZPMAX=ABS(ZP-ZPROF)
             ZPEARL=ABS(B-ZPROF)
             IF (ZPEARL.LE.ZPMAX) THEN
                   VOLPEAR=ZPEARL/LINEAL
             ELSE
                   VOLPEAR=ZPMAX/LINEAL
             END IF
             IF (VOLPEAR.GT.(1-VF)) VOLPEAR=0.99*(1-VF)         
666          VPEARL=VOLPEAR
	     IF (VELFER.GT.VELMAX) VPEARL=0.0
             VTOT=VF+VPEARL
             WRITE(*,1215)B,X1,VPEARL,VTOT
2         CONTINUE
      END IF
C
C Maximum volume fraction of ferrite has just been achieved.
C               Supersaturation case
C
      IF (ZPROF.GE.Z2) THEN
          ZPROF=Z2
          ZVOL=ZPROF*1.0D-02
          VF=2.0D+00*(ZVOL/DGAMMA)
	  VELFER=2.0D-02*DIFF*((Z2-ZPROF)/((LINEAL-ZPROF)**2))
          WRITE(*,1213)ZPROF
          WRITE(*,2022)VELFER
          WRITE(*,1020)VELMAX 
          WRITE(*,1217)VF
          write(*,1214)
          WRITE(*,1216)
          WRITE(*,2023)
          DO 3 B=ZPROF,LINEAL,0.1D-03
             X1=PROFILD(B,XGAG1)
             ZPEARL=LINEAL-Z2
             VOLPEAR=ZPEARL/LINEAL        
             VPEARL=VOLPEAR
	     IF (VELFER.GT.VELMAX) VPEARL=0.0
777          VTOT=VF+VPEARL
             WRITE(*,1215)B,X1,VPEARL,VTOT
3         CONTINUE
      END IF          
C
C ----------------------Format statments only--------------------------
C
C
1152  FORMAT(5X,'Temperature higher than Ae3. No Ferrite nucleation',//)
1800  FORMAT(5X,'Time for nucleation =',f10.2,' seconds',//)
1743  FORMAT(//,10X,'= = = = TIME LOWER NUCLEATION TIME! = = = = ',//)
6010  FORMAT(5x,'Maximum half-thickness in this system =',e14.6,//)
2190  FORMAT(/,15X,'<<<<<< CRITICAL TEMPERATURES >>>>>>',/)
2160  FORMAT(18X,'Ae3 temperature (C)=',f7.2)
2170  FORMAT(18X,'Ae1 temperature (C)=',f7.2)

2200  FORMAT(/,15X,'^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^',/)
2150  FORMAT(5X,'TIMESTEP in calculations=',f7.2)
2130  FORMAT(5x,'BE CAREFUL! No real data ')
2120  FORMAT(5X,'TIMEPROF fitted acording AGS,Z1 & Z2 used')
2110  FORMAT(5X,'New time adjusted to metallographic parameters=',
     & f7.2)
2100  FORMAT(//)
1020  FORMAT(5X,'Highest rate of alpha/gamma to allow nucleation  ',
     & '(m/s)=',e12.4)
2022  FORMAT(5X,'Migration rate of alpha/gamma boundary (m/s)=',e12.4)
2020  FORMAT(/,15X,'No impingement takes place',/)
2021  FORMAT(/,15X,'WARNING!,Impingement is taken place',/)
2023  FORMAT(/,15X,'The growth of ferrite has finished',/)
1218  FORMAT(5X,'Carbon concen.(%wt) at gamma/theta interf. ',
     & ' in austenite=',e14.5)
1217  FORMAT(5X,'Volume fraction of allotromorphic ferrite=',e14.5,/)
1215  format(1x,e14.5,3x,e12.4,3x,e12.4,3X,E12.4)
1214  format(/,4x,'Position(cm)',6x,'C(%wt)',7x,'Vpearlite',8x,'Vtot',
     & /)
1216  format('----------------------------------------------------',
     & '----------',/)
1212  FORMAT(5x,'Holding time at this temperature=',F7.2)
1213  FORMAT(5x,'Position of interface at this temperature=',e14.5)
1210  FORMAT(//)
1211  FORMAT('~~~~~~~~~~~~~~~~~Carbon Profile ahead interface~~~~~~',
     & '~~~~~~~~~~~~~~~~~~~~')
22    FORMAT(/,1X,'Time(s)',4x,'Z-imp.(cm)',4x,'Z-para.(cm)',3x,
     &'CL (%wt)',4x,'Vol.Frac',2x,'Migration Rate(m/s)',/)
23    FORMAT('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
     & '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~')
1208  FORMAT(/,5x,'Impingement began at ',e14.6)
1209  Format(5x,'Maximum half-thickness in Fe-C system =',e14.6)
1207  FORMAT(//,21X,'********End of ferrite growth*********',/)
1206  format(2x,f9.2,1x,e12.4,1x,e12.4,1x,e12.4,1x,e12.4,1x,e12.4)
1205  FORMAT(/,21X,'********Parabolic Growth********',/)
1202  format(//,21x,'********Impingement begins********',//)  
1201  format(//)
106   format(/,5x,'C-C interaction energy in austenite '
     &,' in J/mol=',f8.3)
118   format(5x,'C-C interaction energy in ferrite '
     &,' in J/mol=',f12.3)
107   format(/,5x,'Mole fraction of C in austenite=',f7.5)
108   format(5x,'Concentration of carbon in alpha/gama interface',     
     & ' into ferrite=',f7.5)
111   format(5x,'Parabolic-thickening rate constant (3-D)'
     &,' in cm/s**(1/2) =',E14.5)
1434   format(5x,'Parabolic-thickening rate constant (1-D)'
     &,' in cm/s**(1/2) =',E14.5)
115   FORMAT(5X,'Austenite grain size ,in microns=',f6.2)
105   FORMAT(/,5X,'Isothermal temperature=',f6.1)
121   FORMAT('********************************************************'
     &,'*************')
120   format(5x,'Holding time=',f7.2)
104   FORMAT(//,12X,'****Chemical composition****',///
     &,2x,'Carbon=',f6.4
     &,15x,'Silicon=',f6.4,/,2x,'Manganese=',f6.4,12x,'Nickel=',f6.4
     &,/,2x,'Molibdenon=',f6.4,11x,'Cromium=',f6.4,/,2x,'Vanadium='
     &,f6.4,//)
109   format(5x,'Concentration of carbon in alpha/gama interface',     
     & ' into austenite=',f10.8)
1001  format(/,15x,'~~~~~~~MOLE FRACTION~~~~~~~~')
1002  FORMAT(/,15X,'~~~~~WEIGHT PERCENTAGE~~~~~~')
1011  format(/,5x,'Carbon concentration of carbon at the bulk=',f7.5)
113   FORMAT(//,5X,'Half-thickness of the layer of ferrite ' 
     &,' in cm. NO IMPINGEMENT =',E14.5)
1010  format(/,5x,'C diffusion coefficient into austenite (cm**/s)=',
     & e14.5)
300   FORMAT(5X,'Number of particles per unit area =',D12.4)
8     FORMAT(5X,'DALPHA =',D12.4,' m ',/,5x,'DGAMMA = ',D12.4,' m')
1745  STOP
      END
C
C***********************************************************
C
      SUBROUTINE ALP(XGAG,XBAR,XAGA,DIFF,CTEMP,ALPHA)
C
C Writing suppressed 5 July 1994
C
      IMPLICIT NONE
      DOUBLE PRECISION X2(22),Y2(22),X(60),Y(60),X1(3),Y1(3)
      DOUBLE PRECISION XAGA,XGAG,XBAR,DIFF,CTEMP,ALPHA
	DOUBLE PRECISION GES,OMEGA,DUM1,MAP_UTIL_DAERF,DER
     &, CONST,SLOPE,CORR
	INTEGER I1,I2,I3,J,J1
C
      GES=0.51466
C
C     GES IS A GUESS VALUE OF ALPHA1/(2*SQRT(D))
C
      I3=0
      J=0
C
      OMEGA=(XGAG-XBAR)/(XGAG-XAGA)
C
3     I3=I3+1
      DUM1=GES*DEXP(GES*GES)*(1.0D+00-MAP_UTIL_DAERF(GES))-(OMEGA
     &/DSQRT(3.14159D+00))
      X2(I3)=GES
      Y2(I3)=DUM1
C 
      DER=(DEXP(GES*GES)*(1.0D+00 + 2.0D+00*GES*GES))*
     &(1.0D+00-MAP_UTIL_DAERF(GES))-GES*2.0D+00/(DSQRT(3.14159D+00))
      I1=DINT(GES*1000000)
      GES=GES-DUM1/DER
      IF(GES .GT. 12.0) GOTO 1
      I2=DINT(GES*1000000)
      IF(I1 .EQ. I2)GOTO 2
      IF(I3 .GT. 20)GOTO 1
      GOTO 3
1     GES=0.51466
7     J=J+1
      DUM1=GES*DEXP(GES*GES)*(1.0D+00-MAP_UTIL_DAERF(GES))-(OMEGA
     &/DSQRT(3.14159D+00))
      DER=(DEXP(GES*GES)*(1.0D+00 + 2.0D+00*GES*GES))*
     &(1.0D+00-MAP_UTIL_DAERF(GES))-GES*2.0D+00/(DSQRT(3.14159D+00))
      X(J)=GES
      Y(J)=DUM1
      IF(J .EQ. 1) GOTO 7
      IF(DABS(Y(J)) .GT. DABS(Y(J-1)))GOTO 6
      GES=GES-0.5*DUM1/DER
      GOTO 7
C
C
6     X1(3)=X(J-1)
      X1(2)=X(J-2)
      X1(1)=X(J-3)
      Y1(3)=Y(J-1)
      Y1(2)=Y(J-2)
      Y1(1)=Y(J-3)
C
      CALL ANAL(3,0,CONST,SLOPE,CORR,X1,Y1)
C
      GES=-1.0D+00*CONST/SLOPE
      DUM1=GES*DEXP(GES*GES)*(1.0D+00-MAP_UTIL_DAERF(GES))-(OMEGA
     &/DSQRT(3.14159D+00))
C
      DO 10 J1=1,I3
        IF(DABS(DUM1) .GT. DABS(Y2(J1)))GOTO 9
        GOTO 10
9       GES=X2(J1)
        DUM1=Y2(J1)
10    CONTINUE
C
2     ALPHA=GES*2.0D+00*DSQRT(DIFF)
C      WRITE(4,4)CTEMP,ALPHA,DUM1
4     FORMAT(' CTEMP=', F8.0,10X,' ALPHA1, (cm/(sec)**.5)=',
     &D12.5,   10X,' DUM1=',D12.5/
     &/'******** *********** ********** ************   ***********')
5     FORMAT(2D12.4)
      RETURN
      END
C	
C*********************************************************************
C
      SUBROUTINE ANAL(J7,J9,CONST,SLOPE,CORR,X,Y)
	IMPLICIT NONE
      DOUBLE PRECISION X(40),Y(40)
      DOUBLE PRECISION AX,AX2,AY,AY2,AXY,CONST,CORR,SLOPE
      INTEGER I,J8,J9,J7
C
      J8=J7-J9
      AX=0.0D+00
      AY=0.0D+00
      AX2=0.0D+00
      AY2=0.0D+00
      AXY=0.0D+00
      DO 1 I=1,J8
        AX  = AX+X(I)
        AY  = AY+Y(I)
        AXY = AXY+X(I)*Y(I)
        AX2 = AX2+X(I)*X(I)
        AY2 = AY2+Y(I)*Y(I)
1     CONTINUE
      CONST=(AY*AX2-AX*AXY)/(J8*AX2-AX*AX)
      SLOPE=((J8*AXY-AX*AY)/(J8*AX2-AX*AX))
      CORR=(J8*AXY-AX*AY)/(DSQRT((J8*AX2-AX*AX)*
     &(J8*AY2-AY*AY)))
      WRITE(6,2)CONST,SLOPE,CORR
2     FORMAT('   INTERCEPT=',D12.4,'    SLOPE=',D10.3,
     &'    CORRELATION=',F8.4)
      RETURN
      END
C
C ****************************************************************
C Finds approximate temperature at which paraequilibrium ferrite
C formation first becomes possible
C H. K. D. H. Bhadeshia, 6 July 1994
C
C All temperatures in centigrade.
C
      SUBROUTINE AE3TEM(C,W,T10,T20,TAE3)
C
      IMPLICIT NONE
      DOUBLE PRECISION C(12)
	DOUBLE PRECISION W,W1,T,T10,T20,TAE3,XA,X1,R,H,S,XEQ
     &, KTEMP,H1,S1,F,ENERGY,AJ,AJ1,ETEQ,TEQ,AFEG,DAFEG
	INTEGER I
C
C       write(*,*)c,w,t10,t20
       X1=C(1)
       XA=0.001D+00
       R=8.31432D+00
       W1=48570.0D+00
       H=38575.0D+00
       S=13.48D+00
       XEQ=0.05D+00
C
      DO 1 I=1183,973,-1
       KTEMP=I
       T=KTEMP
C
	IF (T .LE. 1000.0) GOTO 20
	  H1=105525.0D+00
	  S1=45.34521D+00
	GOTO 19
20      H1=111918.0D+00
	  S1=51.44D+00
C
19    F=ENERGY(T,T10,T20)
      AJ=1.0D+00-DEXP(-W/(R*T))
      AJ1=1.0D+00-DEXP(-W1/(R*T))
51    TEQ=R*T*AFEG(XEQ,AJ)-F
C       WRITE (*,*) T, F, XEQ, X1, TEQ
      IF (DABS(TEQ) .LT. 1.0) GOTO 52
      ETEQ=DAFEG(XEQ,AJ)*R*T
      XEQ=XEQ-TEQ/ETEQ
      GOTO 51
52    IF(XEQ .GE. X1) THEN
	  TAE3=KTEMP-273.15D+00
	  GOTO 222
      END IF
1     CONTINUE
C
222   CONTINUE
      RETURN
      END
C
C***********************************************************************
C Finds approximate temperature at which paraequilibrium Pearlite
C formation first becomes possible.
C Based on the interface composition reach the eutectoid concentration
C C.Capdevila-Montes. (Cambridge,1997)
c
C Pearlite temperatures added according to:
C	A.K Sinha.'Ferreous Physical Metallurgy'. Ed:Butterworths
C	USA,1989. pp.202.
C
C C.Capdevila-Montes. (Cambridge,1997)
C
C All temperatures in centigrades.
C
      SUBROUTINE AE1TEM(WT,W,T10,T20,TAE1)
C
      IMPLICIT NONE
      DOUBLE PRECISION CE(12),WT(12),XEU(6)
	DOUBLE PRECISION W,T,T10,T20,TAE1,SUM,XEUTEC,XA
     &, W1,H,H1,S,S1,XEQ,KTEMP,AFEG,DAFEG,ETEQ,F,ENERGY
     &, AJ,AJ1,R,TEQ
	INTEGER I
C
C Influence of the differents alloy elements on eutectoid concentration
C
       XEU(1)=-0.68D-2*(WT(5)**3)+0.105D+00*(WT(5)**2)-0.4717D+00
     & *WT(5)+0.8089D+00
       XEU(2)=-0.16D-02*(WT(11)**3)+0.0442D+00*(WT(11)**2)-0.3127D+00
     & *WT(11)+0.8289D+00
       XEU(3)=-5D-05*(WT(2)**3)+0.0129D+00*(WT(2)**2)-0.1467D+00*
     & WT(2)+0.8161D+00
       XEU(4)=-5D-03*(WT(6)**3)+0.0116D+00*(WT(6)**2)-0.1058D+00*
     & WT(6)+0.8064D+00
       XEU(5)=0.17D-02*(WT(3)**3)-0.013D+00*(WT(3)**2)-0.0428D+00*
     & WT(3)+0.8082D+00
       XEU(6)=-0.2D-3*(WT(4)**3)-2.0D-15*(WT(4)**2)-0.0192D+00*WT(4)
     & +0.8
C
C Eutectoid concentration
C
       SUM=0.0
       DO 11 I=1,6
	  SUM=SUM+XEU(I)
11     CONTINUE
       XEUTEC=SUM/6
C
       WT(1)=XEUTEC
       CALL CONV(1,WT,CE)
       XEUTEC=CE(1)
       XA=0.001D+00
       R=8.31432D+00
       W1=48570.0D+00
       H=38575.0D+00
       S=13.48D+00
       XEQ=0.05D+00

C
      DO 1 I=1183,873,-1
       KTEMP=I
       T=KTEMP
C
	IF (T .LE. 1000.0) GOTO 20
	  H1=105525.0D+00
	  S1=45.34521D+00
	GOTO 19
20      H1=111918.0D+00
	  S1=51.44D+00
C
19    F=ENERGY(T,T10,T20)
      AJ=1.0D+00-DEXP(-W/(R*T))
      AJ1=1.0D+00-DEXP(-W1/(R*T))
51    TEQ=R*T*AFEG(XEQ,AJ)-F
C       WRITE (*,*) T, F, XEQ, X1, TEQ
      IF (DABS(TEQ) .LT. 1.0) GOTO 52
      ETEQ=DAFEG(XEQ,AJ)*R*T
      XEQ=XEQ-TEQ/ETEQ
      GOTO 51
52    IF (XEQ .GE. XEUTEC) THEN
	  TAE1=KTEMP-273.15D+00
	  GOTO 222
      ENDIF
c      write(*,*)teq,xeq,(ktemp-273.15d+00)
1     CONTINUE   
222   RETURN
      END
C
C***********************************************************************
C
C Selects appropriate time step for calculation 
C AGS in (cm)
C C.Capdevila-Montes. (Cambridge,1997)
C
	SUBROUTINE STEP(AGS,TS)
	IMPLICIT NONE
	DOUBLE PRECISION AGS,TS
C
	IF (AGS.GE.200.0D-04) TS=20.0D+00
	IF (AGS.GE.40.0D-04.AND.AGS.LT.200.0D-04) TS=10.0D+00
	IF (AGS.GE.20.0D-04.AND.AGS.LT.40.0D-04) TS=5.0D+00
	IF (AGS.GE.6.0D-04.AND.AGS.LT.20.0D-04) TS=1.0D+00
	IF (AGS.GE.2.0D-04.AND.AGS.LT.6.0D-04) TS=0.1D+00
	RETURN
	END 
C***********************************************************************
C
C Therefore, the restriction that the migration of the alpha / gamma boundary, 
C at which nucleation takes place must not exceed that which displaces this 
C boundary a distance of one lattice parameter, a, in the time required for 
C an embryo to develop to the critical nucleus size should be satisfied.
C
C SUBROUTINE TO CALCULATE THE CRITICAL ALPHA/GAMMA BOUNDARY MOVING RATE
C
C Thermodynamics parameters from Zener calculations.
C Parameter of austenite according M.Takahashi.
C
C Units: VEL (m/s). S.I.
C 
C Bibliography:
C -------------
C 		Aaronson, et al, Met. Trans., vol 9A, 1978, pp.363.
C		Zener,C, TRans. AIME, 1946, pp.550
C		Takahashi,M, 1992, PhD. University of Cambridge
C		Ridley,N, Stuart,H and Zwell,L, Trans. AIME, 1969, 245, 1834
C		Dyson,D.J and Holmes,B,JISI,1970,May,469.
C		Leslie, W.C, "The physical metallurgy of steels",
C		            McGraw-Hill,1981,pp.256.
C 
C Varibles:
C ---------
C	INTERFE->Interface energy
C	EXG-> Thermal Expansion of austenite (experimental)
C	GV-> Change of Free Energy
C	HEUTEC->Enthalpy of eutectoid transformation
C	GG->Austenite parameter
C 
C C.Capdevila-Montes. (Cambridge,1997)
C
      SUBROUTINE CRITICAL(CTEMP,AE1,XG,D,C,WT,VEL)
      IMPLICIT NONE
	DOUBLE PRECISION C(11),WT(11)
      DOUBLE PRECISION CTEMP,T,XG,D,VEL,KK,GV,EINTERF,K
     &, GG,GG0,PI,NUM,DENOM,FAC,D1,HEUTEC,TEUTEC
     &, EXG,AE1,PHI      
C
C Data: (SI units)
C
      PARAMETER (PI=3.1416D+00)
      KK=13.80658D-24
      EINTERF=0.065D+00
      EXG=0.227D-04
      K=1.0D-05
      T=CTEMP+273.0D+00
C
C Thermodynamics calculations for Free energy of nucleation of cementite
C HEUTEC in cal/cm**3 , TEUTEC in centigrades and GV in cal/cm**3
C Eutectoid temperature according to Leslie.
C
      HEUTEC=165D+00
      TEUTEC=AE1 
      GV=HEUTEC*((TEUTEC-CTEMP)/TEUTEC)
C
C SI units
C
      GV=4.19D+06*GV
      D1=1.0D-04*D
      PHI=60.0D+00
C 
C Lattice parameter of austenite. Based on M.Takahashi PhD., Ridley,1969
C and Dyson and Holmes,1970.
C
      GG0=3.573D+00+6.5D-03*C(1)+1.0D-03*C(3)-0.2D-03*C(4)+
     & 0.6D-03*C(6)+2.8D-03*C(10)-0.4D-03*C(8)+1.4D-03*C(9)+
     & 5.3D-03*C(5)+1.7D-03*C(7)+5.7D-03*C(11)
C
C GG0 in amstromgs. GG in meters
C
      GG=GG0*(1.0D+00+(CTEMP-25.0D+00)*EXG)*1.0D-10
C
C The highest growth rate
C
      NUM=(GG**3)*D1*XG*(GV**3)
      FAC=PI*KK*T*(EINTERF**5)*K
      DENOM=16.0D+00*(1-DCOS(PHI))*SQRT(FAC)
      VEL=NUM/DENOM
C
C VEL in m/s
C
      RETURN
      END
C
C***********************************************************************
C
C  FUNCTION TO CALCULATE THE DISTRIBUTION OF CARBON AHEAD  INTERFACE
C  IN ABSENCE OF IMPINGEMENT.
C  C.CAPDEVILA MONTES (Cambridge, 1997)
C

      DOUBLE PRECISION FUNCTION PROFILA(X,C2,C1,C3,Z)
	IMPLICIT NONE
      DOUBLE PRECISION X,C2,C1,C3,Z,Y,M,C
C
      Y=2.0D+00*Z*((C2-C3)/(C1-C2))
      M=(C1-C2)/Y
      C=C2+M*(Y+Z)
C
C Ecuation of the carbon profile ahead the interface based on Zener's 
C linear approximation. The carbon depleted from the ferrite and the 
C area under the profile should be the same. Conservation of carbon in
C the diffusion process.
C
      PROFILA=-M*X+C
      IF (PROFILA.LE.C2) PROFILA=C2
      RETURN
      END  
C
C*********************************************************************
C  FUNCTION TO CALCULATE THE DISTRIBUTION OF CARBON AHEAD  INTERFACE
C  TAKEN IN ACCOUNT THE EFFECT OF IMPINGEMENT.
C  C.CAPDEVILA MONTES (Cambridge, 1997)
C
      DOUBLE PRECISION FUNCTION PROFILD(X,xconc)
	IMPLICIT NONE
      DOUBLE PRECISION X,XCONC
      PROFILD=XCONC
      RETURN
      END 
C
C*********************************************************************
C  FUNCTION TO CALCULATE THE DISTRIBUTION OF CARBON AHEAD  INTERFACE
C  TAKEN IN ACCOUNT THE EFFECT OF IMPINGEMENT.
C  C.CAPDEVILA MONTES (Cambridge, 1997)
C
      DOUBLE PRECISION FUNCTION PROFILB(X,C2,C1,AGS,Z)
	IMPLICIT NONE
      DOUBLE PRECISION X,C2,C1,AGS,Z,B,TG
      TG=((C1-C2)/(AGS-Z))
      B=C1
      PROFILB=-TG*(X-Z)+B
      RETURN
      END      
C*****************************************************************
C Copyright Dr. H. K. D. H. Bhadeshia, University of Cambridge
C MUCG1.FOR
C calculation of XEQ and GMAX only. Adapted from MUCG46.FOR
C 5 July 1994
C
C Department of Materials Science and Metallurgy, Pembroke St. Cambridge CB2 3QZ
C
C
      SUBROUTINE AG(C,W,T10,T20,KTEMP,GMAX)
      IMPLICIT NONE
      DOUBLE PRECISION C(12)
      DOUBLE PRECISION W,W1,T,T10,T20,KTEMP,GMAX,X1,XA,R,F
     & ,H,H1,S,S1,A,A1,AJ,AJ1,XEQ,ETEQ,AFEG,DAFEG,ENERGY,AEQ
     & ,TG,CTEMP,CG,TEQ,AFEQ,AFE,X
C
      X1=C(1)
      XA=0.001D+00
      R=8.31432D+00
      W1=48570.0D+00
      H=38575.0D+00
      S=13.48D+00
      XEQ=0.1D+00
C
C
      T=KTEMP
      CTEMP=T-273.00D+00
C
      IF (T .LE. 1000.0) GOTO 20
      H1=105525.0D+00
      S1=45.34521D+00
      GOTO 19
20    H1=111918.0D+00
      S1=51.44D+00
C
19    F=ENERGY(T,T10,T20)
      AJ=1.0D+00-DEXP(-W/(R*T))
      AJ1=1.0D+00-DEXP(-W1/(R*T))
51    TEQ=R*T*AFEG(XEQ,AJ)-F
      IF (DABS(TEQ) .LT. 1.0) GOTO 52
      ETEQ=DAFEG(XEQ,AJ)*R*T
      XEQ=XEQ-TEQ/ETEQ
      GOTO 51
52    AEQ=CG(XEQ,T,W,R)
      AFEQ=AFEG(XEQ,AJ)
      A=CG(X1,T,W,R)
      AFE=AFEG(X1,AJ)
C
      CALL GMAAX(A1,A,W1,F,R,T,X,AFE,H1,S1)
      GMAX=R*T*(A1-A)
C
C
      RETURN
      END
C
C***********************************************************************
C CALCULATION OF THE OPTIMUM NUCLEUS C CONTENT AND ACTIVITY OF C IN
C FERRITE NUCLEUS
C
      SUBROUTINE GMAAX(A1,A,W1,F,R,T,X,AFE,H1,S1)
      IMPLICIT NONE
	DOUBLE PRECISION A,A1,AJ1,W1,F,R,T,X,AFE,H1,S1,D1
     & ,B1,B2,B3,A1FE,TEST,DA1,DA2,DA1FE,ERROR
C
C Estimate a value for X
C February 1991
C
       X=6.3998D-07*T-3.027D-04
       IF(X .LT. 0.1D-07)X=1.0D-08
       IF(X .GT. 0.32D-03)X=0.32D-03
C
C Estimation complete
C
      AJ1=1.0D+00-DEXP(-W1/(R*T))
1     D1=DSQRT(9.0D+00-6.0D+00*X*(2.0D+00*AJ1+3.0D+00)
     &+(9.0D+00+16.0D+00*AJ1)*X*X)
      B1=((D1-3.0D+00+5.0D+00*X)/(D1+3.0D+00-5.0D+00*X))
      B2=((3.0D+00-4.0D+00*X)/X)
      B3=(H1 - S1*T + 4.0D+00*W1)/(R*T)

      A1=DLOG(B1*B1*B1*B1*B2*B2*B2)+B3
C
      IF(X .GT. 1.0D-08)THEN
        A1FE=DLOG(1.0D+00-X)
      ELSE
        A1FE=X
      ENDIF
C
      TEST=F+R*T*(A1FE-AFE) - R*T*(A1-A)
C
C Newton iteration
C
      IF (DABS(TEST) .GT. 10.0) THEN
        DA1=(3.0D+00*X/(3.0D+00-4.0D+00*X))*(
     &  (4.0D+00*X-3.0D+00)/(X*X)-4.0D+00/X)
        DA2=(0.5D+00/D1)*(-12.0D+00*AJ1-18.0D+00+
     &  18.0D+00*X+32.0D+00*AJ1*X)
        DA2=4.0D+00*(((DA2+5.0D+00)/(D1-3.0D+00+5.0D+00*X))
     &  -((DA2-5.0D+00)/(D1+3.0D+00-5.0D+00*X)))
        DA1=DA1+DA2
        DA1FE=1.0D+00/(X-1.0D+00)
        ERROR=TEST/(R*T*(DA1FE-DA1))
        IF (ERROR .GT. X) ERROR = 0.3D+00*X
        X=DABS(X-ERROR)
        GOTO 1
      ENDIF
C End of iteration
      RETURN
      END
CC
C*********************************************************************
C  FUNCTION FOR CALCULATE CONCENTRATION OF CARBON IN THE MIDDLE OF 
C  AUSTENITE GRAIN
C  Based on Gilmour et al , Met. Trans. 1972, vol.3, pp.3215.
C
C C.Capdevila-Montes. (Cambridge,1997)
c
      DOUBLE PRECISION FUNCTION CL(X1,X2,AGS,X)
	IMPLICIT NONE
      DOUBLE PRECISION X,X1,X2,AGS
      CL=((2*AGS*X1)-(X2*(AGS-X)))/(AGS-X)
      RETURN
      END
C*********************************************************************
C Subroutine to write the interface composition in weight percentage.
C C.Capdevila-Montes. (Cambridge,1997)
c
      SUBROUTINE INTERF(CARBON,X1,X2,C,Y1,Y2)
      IMPLICIT NONE
      DOUBLE PRECISION N(7),M(7),W(7),C(7)
      DOUBLE PRECISION CARBON,X1,X2,Y1,Y2
	WRITE(*,1002)
      write(*,1011)CARBON      
      M(1)=X1
      M(2)=C(2)
      M(3)=C(3)
      M(4)=C(4)
      M(5)=C(5)
      M(6)=C(6)
      M(7)=C(7)
      CALL CONV(0,W,M)
      Y1=W(1)
      write(*,109)Y1
      N(1)=X2
      N(2)=C(2)
      N(3)=C(3)
      N(4)=C(4)
      N(5)=C(5)
      N(6)=C(6)
      N(7)=C(7)
      CALL CONV(0,W,N)
      Y2=W(1)
      write(*,108)Y2
1002  FORMAT(/,15X,'~~~~~WEIGHT PERCENTAGE~~~~~~')
109   format(5x,'Concentration of carbon in alpha/gamma interface',     
     & ' into austenite=',f10.8)
1011  format(/,5x,'Carbon concentration of carbon at the bulk=',f7.5)
108   format(5x,'Concentration of carbon in alpha/gamma interface',     
     & ' into ferrite=',f7.5)
      RETURN
      END
C
C**********************************************************************
C Subroutine to determine parabolic rate constant using 4th order
C polynomial, fitted to calculated values of DLOG(GES) for OMEGA from 0
C to 1.
C S.Parker 28/6/95 from polyfit.for, using all.for and mreg4.f
C Based on 'Allotriomorphic and Idiomorphic Ferrite' Dr. R.C. Reed.
C MODELLING PHASE TRANSFORMATIONS IN STEELS. University of Cambridge-
C Programme for Industry. March 1993.
C
	SUBROUTINE PARABOLIC(XBAR,XGAG,XAGA,DIFF,CTEMP,ALPHA)
	IMPLICIT NONE
	DOUBLE PRECISION ALPHA,GESLOG,OMEGA
	DOUBLE PRECISION XBAR,XGAG,XAGA,DIFF,CTEMP
C
C For a 3-D growth ratae, the parabolic rate is
C
	  OMEGA=(XBAR-XGAG)/(XAGA-XGAG)
        GESLOG=(-0.5701D+01*(OMEGA**4.0d+00))+(0.1768D+02*(OMEGA
     &  **3.0D+00))-(0.1742d+02*OMEGA*OMEGA)+(0.9196D+01*OMEGA)
     &  -2.068
C
C For a 1-D growth rate, the parabolic rate is
C
c	GESLOG=(-0.1380D+02*(OMEGA**4.0d+00))+(0.3676D+02*(OMEGA
c     &	**3.0D+00))-(0.3395d+02*OMEGA*OMEGA)+(0.1620D+02*OMEGA)-4.1989
	ALPHA=DEXP(GESLOG)*2.0D+00*DSQRT(DIFF)
 	RETURN
	END
C
C **********************************************************************
C
      SUBROUTINE ALPH(W,XBAR,CTEMP,XAGA,XGAG,ALPHA,ANS)
C
C Modified to suppress writing. 5 July 1994
C
C 27 September 1985 same as .CONCDIF:RATE, BUT modified to
C allow easier low T calculations and provide iteration error messages
C TYPICAL DATASET
C 780 0.0100
C 740 0.0176
C 700 0.0266
C 660 0.0389
C 600 0.0552
C Modified 24-6-85 to include XINCR
C - this limits no. of Diff coeffs calculated for T<700C
C PROGRAM TO CALCULATE PARABOLIC RATE CONSTANT, 4-1-84
C PROGRAM MODIFIED, 30-8-83, FOR ACCURATE XAPLPHA
C PROGRAM TO CALCULATE THE EFFECTIVE DIFFUSIVITY OF CARBON IN
C AUSTENITE, TAKING ACCOUNT OF THE FACT THAT THIS DIFFUSIVITY
C IS CONCENTRATION DEPENDENT.  USES SILLER AND MCLELLAN THEORY
C TO EXPRESS THE CONCENTRATION DEPENDENCE, AND LACHER ET AL
C THEORY TO ALLOW FOR THE EFFECT OF SUBSTITUTIONAL ALLOYING ELEMENTS
C ON THE ACTIVITY AND W OF CARBON IN AUSTENITE. **7 OCTOBER 1981**
C TEMPERATURE IS READ IN IN DEGREES CENTIGRADE
C CARBON CONTENT READ IN AS MOL FRAC, EXCEPT FOR THE SUBROUTINE OMEGA,
C WHERE THE CARBON CONTENT AND ALLOYING CONTENT IS READ IN AS WT PCT IN
C THE FIRST LINE OF THE DATASET.
C HH=PLANCKS CONST.JOULES/SEC,KK=BOLTZMANNS CONST.JOULES/DEGREE KELVIN
C     D=DIFFUSIVITY OF CARBON IN AUSTENITE
C     Z=COORDINATION OF INTERSTIAL SITE
C     PSI=COMPOSITION DEPENDENCE OF DIFFUSION COEFFICIENT
C     THETA=NO. C ATOMS/ NO. FE ATOMS
C     ACTIV=ACTIVITY OF CARBON IN AUSTENITE
C     R=GAS CONSTANT
C     X=MOLE FRACTION OF CARBON
C     T=ABSOLUTE TEMPERATURE
C     SIGMA=SITE EXCLUSION PROBABLITY
C     W=CARBON CARBON INTERACTION ENERGY IN AUSTENITE
C
      IMPLICIT NONE
      DOUBLE PRECISION DIFF(500), CARB(500)
      DOUBLE PRECISION HH,KK,A5,Z,R,T,DASH,XBAR,XGAG,XAGA,ALPHA,ANS
     &, W,XINCR,X,ACTIV,DACTIV,CG,DCG,SIGMA,PSI,THETA,CTEMP
      INTEGER II,II2,II3
C  W,XBAR,CTEMP,XAGA,XGAG,ALPHA,ANS
      PARAMETER (R=8.31432D+00)
	HH=6.6262D-34
      KK=1.38062D-23
      Z=12
      A5=1.0D+00
      
      T=CTEMP+273.00D+00
      II2=0
      DASH=(KK*T/HH)*DEXP(-(21230.0D+00/T))*DEXP(-31.84D+00)

      DO 9   II=1,1000
	CARB(1)=XBAR
	IF (II .LE. 1) GOTO 8
1     		IF(((XGAG-XBAR) .LT. 0.005) .OR.
     &			(XGAG-CARB(II-1).LT.0.001)) THEN
C	2nd criterion for IF statement added 13/1/97 to improve
C	accuracy of end test IF (CARB(II).GT.XGAG)
		XINCR=0.0001D+00
      		ELSE
		XINCR=0.001D+00
		ENDIF
4     	CARB(II)=CARB(II-1)+XINCR
      	IF (CARB(II) .GT. XGAG) GOTO 5
8     	X=CARB(II)
      	II2=II2+1
      	THETA=X/(A5-X)
      	ACTIV=CG(X,T,W,R)
      	ACTIV=DEXP(ACTIV)
      	DACTIV=DCG(X,T,W,R)
      	DACTIV=DACTIV*ACTIV
      	DACTIV=DACTIV*A5/((A5+THETA)**2)
      	SIGMA=A5-DEXP((-(W))/(R*T))
      	PSI=ACTIV*(A5+Z*((A5+THETA)/(A5-(A5+Z/2)*THETA+(Z/2)*(A5+Z/2)*
     &	(A5-SIGMA)*THETA*THETA)))+(A5+THETA)*DACTIV
      	DIFF(II)=DASH*PSI
9     CONTINUE

5     II3=0

      CALL TRAPE(CARB,DIFF,ANS,II2)
      IF (XGAG .GT. XBAR) THEN
         ANS=ANS/(XGAG-XBAR)
      ELSE 
         ANS=ANS/(XBAR-XGAG)
      ENDIF
	if (ans .lt. 0.0) ans=0.0
C
C calculate 3-D rate constant
C
	CALL PARABOLIC(XBAR,XGAG,XAGA,ANS,CTEMP,ALPHA)
c      write(*,*)ans
      RETURN
      END
C
C**********************************************************************
C Subroutine based on MUCG46.FOR.Copyright Dr. H. K. D. H. Bhadeshia,
C University of Cambridge
C Department of Materials Science and Metallurgy, Pembroke St.
C Cambridge CB2 3QZ C Version 1.7 July 1993 CG modified
C Subroutine used to calculate the equilibrium concentration on the
C interface into austenite (XEQ).
C
C C.Capdevila-Montes. (Cambridge,1997)
C
      SUBROUTINE CONCEN(CTEMP,T10,T20,W,XEQ)
      IMPLICIT NONE
      DOUBLE PRECISION CTEMP,W,XEQ,AJ,F,TEQ,ETEQ,T10,T20,T,R
     &, AFEG,DAFEG,ENERGY
C
      PARAMETER (R=8.31432D+00)
      T=CTEMP+273.0D+00
      F=ENERGY(T,T10,T20)
      AJ=1.0D+00-DEXP(-W/(R*T))
      XEQ=0.1D+00
51    TEQ=R*T*AFEG(XEQ,AJ)-F
c      write(*,*)TEQ,aj,xeq,t
      IF (DABS(TEQ) .LT. 1.0) GOTO 52
      ETEQ=DAFEG(XEQ,AJ)*R*T
      XEQ=XEQ-(TEQ/ETEQ)
      GOTO 51
C
52    RETURN
      END
C
C**********************************************************************
C
      SUBROUTINE NEWOMEGA(C,W,XBAR,T10,T20,ID)
C----------------------------------------------------------------------
C SUBROUTINE TO CALCULATE THE CARBON CARBON INTERACTION ENERGY IN
C AUSTENITE, AS A FUNCTION OF ALLOY COMPOSITION.  BASED ON .MUCG18
C THE ANSWER IS IN JOULES PER MOL.   **7 OCTOBER 1981**
C Feed in the alloy concentration in the following order
C  C    SI   MN   NI   MO   CR   V    CO   CU   AL    W      FE
C       In this Program first the Weight % is converted into Mol fraction
C----------------------------------------------------------------------
C
C  1.   Y() contains only the at% of substitutional considering alone
C
C       T10 and T20 is calculated for the alloy concentration based on
C       Y()
C  2.   Then C-C interaction (W) is calculated based on the ploynomials
C       derived
C       Trial and Error for the interaction energies for Al and Cu
C       MODIFIED FOR OTHER ALLOYING ELEMENTS BY SURESH BABU - C - C
C       APPROX
C       THE CARBON - CARBON INTERACTION MISSING FOR CU AND AL
C----------------------------------------------------------------------
C
      IMPLICIT NONE
      DOUBLE PRECISION C(12),D(12),P(12),Y(12)
	DOUBLE PRECISION B1,B2,B3,T10,T20,W,XBAR,THENW
	INTEGER I,ID,IU,NCO
C
      B3=0.0D+00
      IF (ID .GE. 2) GOTO 2222
      NCO = 12
      IF (NCO .GT. 12) STOP
      D(12)=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)+C(8)+C(9)+C(10)+C(11)
      D(12)=100.0D+00-D(12)
      D(12) =D(12)/55.84D+00
      D(1)  =C(1)/12.0115D+00
      D(2)  =C(2)/28.09D+00
      D(3)  =C(3)/54.94D+00
      D(4)  =C(4)/58.71D+00
      D(5)  =C(5)/95.94D+00
      D(6)  =C(6)/52.0D+00
      D(7)  =C(7)/50.94D+00
      D(8)  =C(8)/58.94D+00
      D(9)  =C(9)/63.54D+00
      D(10) =C(10)/26.98D+00
      D(11) =C(11)/183.85D+00
      B1=D(1)+D(2)+D(3)+D(4)+D(5)+D(6)+D(7)+D(8)+D(9)+D(10)+D(11)+D(12)
C------------------------
C------------------------
      DO 106 IU=1,12
        D(IU)=D(IU)/B1
106   CONTINUE
C------------------------
2222  DO 107 IU=2,11
        Y(IU)=D(IU)/D(12)
107   CONTINUE
2223  XBAR=D(1)
      XBAR=DINT(10000.0D+00*XBAR)
      XBAR=XBAR/10000
C----------------------------------------------------------------------
C       Delta T_nm and Delta T_mag were taken for Co from
C       Aaronson, Pound and Domain work - Trans.Met.Soc. AIME,
C       Vol.236 May, 1966, p. 769   (S. Babu, 18.7.1989)
C       C    SI   MN   NI   MO   CR   V    CO   CU   AL    W      FE
C       1    2    3    4    5    6    7    8    9    10    11     12

C----------------------------------------------------------------------
      T10=Y(2)*(-3)+Y(3)*2+Y(4)*12+Y(5)*(-9)+Y(6)*(-1)+Y(7)*(-12)+
     &    Y(8)* 3.5 +Y(9)*7+Y(10)*(-7)+Y(11)*(-9)
C
C     T10 coefficients are actually Delt_mag - DelT_non.mag
C     T20 coefficients are actually Delt_mag
      T20=-  3*Y(2)-37.5*Y(3)-6*Y(4) -26*Y(5)-19*Y(6)-44*Y(7)+
     &    19.5*Y(8)- 4.5*Y(9)+8*Y(10)-26*Y(11)
C----------------------------------------------------------------------
      P(2)=2013.0341+763.8167*D(2)+45802.87*D(2)**2-280061.63*D(2)**3
     &+3.864D+06*D(2)**4-2.4233D+07*D(2)**5+6.9547D+07*D(2)**6
      P(3)=2012.067-1764.095*D(3)+6287.52*D(3)**2-21647.96*D(3)**3-
     &2.0119D+06*D(3)**4+3.1716D+07*D(3)**5-1.3885D+08*D(3)**6
      P(4)=2006.8017+2330.2424*D(4)-54915.32*D(4)**2+1.6216D+06*D(4)**3
     &-2.4968D+07*D(4)**4+1.8838D+08*D(4)**5-5.5531D+08*D(4)**6
      P(5)=2006.834-2997.314*D(5)-37906.61*D(5)**2+1.0328D+06*D(5)**3
     &-1.3306D+07*D(5)**4+8.411D+07*D(5)**5-2.0826D+08*D(5)**6
      P(6)=2012.367-9224.2655*D(6)+33657.8*D(6)**2-566827.83*D(6)**3
     &+8.5676D+06*D(6)**4-6.7482D+07*D(6)**5 +2.0837D+08*D(6)**6
      P(7)=2011.9996-6247.9118*D(7)+5411.7566*D(7)**2
     &+250118.1085*D(7)**3-4.1676D+06*D(7)**4
C----------------------------------------------------------------------
C           CO  CU   AL    W      FE
C           8   9    10    11     12
C           EFFECT TAKEN FROM HKDB - MET.SCI - 1981
C           RESULT IN J/MOLE    (S. Babu, 18.7.1989)
C           W SAME AS MOLYBDENUM  IN INTERACION ENERGY
C----------------------------------------------------------------------
      P(8)=(8427.00+5986*D(8))/4.187
      P(9)=  2011.0
      P(10)= 2011.0
      P(11)= 2006.834-2997.314*D(5)-37906.61*D(5)**2+1.0328D+06*D(5)**3
     &-1.3306D+07*D(5)**4+8.411D+07*D(5)**5-2.0826D+08*D(5)**6
      B2=0.0D+00
      DO 108 IU=2,11
        B3=B3+P(IU)*Y(IU)
        B2=B2+Y(IU)
108   CONTINUE
      IF (B2 .EQ. 0.0D+00) THEN
       		W=8054.0
      ELSE
     	        W=(B3/B2)*4.187
      ENDIF
      IF (B3 .EQ. 0.0) THEN W = 8054.0
      CONTINUE
      DO 999 I = 1, 12
        C(I) = D(I)
999   CONTINUE
      RETURN
      END
C
C****************************************************************
C     ENERGY ENERGY
      DOUBLE PRECISION FUNCTION ENERGY(T,T10,T20)
	IMPLICIT NONE
      DOUBLE PRECISION T,T10,T20,T7,G,GC
      T7=T-100*T20
      CALL GALGA(G,GC,T7)
      ENERGY=(141.0*T10*4.187) - G                                                                            
      RETURN
      END
C
C*****************************************************************
C FUNCTION GIVING LFG  LN(ACTIVITY) OF CARBON IN AUSTENITE
      DOUBLE PRECISION FUNCTION CG(X,T,W,R)
	IMPLICIT NONE
      DOUBLE PRECISION J,DG,DUMMY,T,R,W,X
      J=1-DEXP(-W/(R*T))
      DG=DSQRT(1-2*(1+2*J)*X+(1+8*J)*X*X)
      DUMMY=5*DLOG((1-2*X)/X)+6*W/(R*T)+((38575.0)-(
     &13.48)*T)/(R*T)
      CG=DUMMY+DLOG(((DG-1+3*X)/(DG+1-3*X))**6)
      RETURN
      END
C
C**********************************************************************
C
C FUNCTION GIVING DIFFERENTIAL OF LN(ACTIVITY) OF CARBON IN AUSTENITE, L
C DIFFERENTIAL IS WITH RESPECT TO X
      DOUBLE PRECISION FUNCTION DCG(X,T,W,R)
	IMPLICIT NONE
      DOUBLE PRECISION J,DG,DDG,X,T,W,R
      J=1-DEXP(-W/(R*T))
      DG=DSQRT(1-2*(1+2*J)*X+(1+8*J)*X*X)
      DDG=(0.5/DG)*(-2-4*J+2*X+16*J*X)
      DCG=-((10/(1-2*X))+(5/X))+6*((DDG+3)/(DG-1+3*X
     &)-(DDG-3)/(DG+1-3*X))
      RETURN
      END
C
C**********************************************************************

C        SUBROUTINE TRAPE
C
C        PURPOSE
C           TO COMPUTE THE VECTOR OF INTEGRAL VALUES FOR A GIVEN
C           GENERAL TABLE OF ARGUMENT AND FUNCTION VALUES.
C
C        USAGE
C             CALL TRAPE (X,Y,Z,NDIM)
C
C        DESCRIPTION OF PARAMETERS
C           X      - DOUBLE PRECISION INPUT VECTOR OF ARGUMENT VALUES.
C           Y      - DOUBLE PRECISION INPUT VECTOR OF FUNCTION VALUES.
C           Z      - THE RESULTING DP. VECTOR OF INTEGRAL VALUES. Z
C                    MAYBE
C                    IDENTICAL WITH X OR Y.
C           NDIM   - THE DIMENSION OF VECTORS X,Y,Z. NDIM MAX. 1000
C
C        REMARKS
C           NO ACTION IN CASE NDIM LESS THAN 1.
C
C        SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C           NONE
C
C        METHOD
C           BEGINNING WITH Z(1)=0, EVALUATION OF VECTOR Z IS DONE BY
C           MEANS OF TRAPEZOIDAL RULE (SECOND ORDER FORMULA).
C           FOR REFERENCE, SEE
C           F.B.HILDEBRAND, INTRODUCTION TO NUMERICAL ANALYSIS,
C           MCGRAW-HILL, NEW YORK/TORONTO/LONDON, 1956, PP.75.
C
C     ..................................................................
C
      SUBROUTINE TRAPE(X,Y,ANS,NDIM)
C
      IMPLICIT NONE
      DOUBLE PRECISION X(10000),Y(10000),AZ(10000)
      DOUBLE PRECISION SUM1,SUM2,ANS
      INTEGER I,NDIM
C
      SUM2=0.D+00
      IF(NDIM-1)4,3,1
C
C     INTEGRATION LOOP
1     DO 2 I=2,NDIM
        SUM1=SUM2
        SUM2=SUM2+.5D+00*(X(I)-X(I-1))*(Y(I)+Y(I-1))
        AZ(I-1)=SUM1
2     CONTINUE
3     AZ(NDIM)=SUM2
      ANS=SUM2
C
4     CONTINUE
      RETURN
      END
C
C ***********************************************************************
C
      SUBROUTINE GALGA (G,GC,T)
C
C Subroutine to calculate pure Fe's difference in Gibbs Free Energy for
C the reaction ferrite -> austenite, Delta G_o Fe{ferrite to
C austenite}.
C Data due to Kaufman, Clougherty and Weiss, Acta Met. 11 1963 p323.
C Their data used here, with corrections. Range 0-1810K.
C This set of polynomials: R.C.Reed, 30th April 1989,
C written up in RCR10.AABS1.GALGA, with graphs.
C NB Input T in degrees Kelvin, output GC is in J/mol.
C NNB This routine will return a value above the melting point of pure
C iron.
      IMPLICIT NONE
      DOUBLE PRECISION G,GC,T
C
      IF (T.LT.3.56D+02) THEN
	G =  0.13132885D+04 + 1.3141025D-02*T - 2.580128186D-03*T*T
      ENDIF
C
      IF (T.GT.3.56D+02 .AND. T.LT.4.28D+02) THEN
	G =  1.1224052D+03 + 3.542835682*T - 2.2496774D-02*T*T
     &     + 4.326701963D-05*T*T*T - 3.072096339D-08*T*T*T*T
      ENDIF
C
      IF (T.GE.4.28D+02 .AND. T.LT.6.34D+02) THEN
	G = - 1.7031057D+04 + 0.1448353906D+03*T - 0.428166357*T*T
     &      + 5.50871582D-04*T*T*T - 2.633397102D-07*T*T*T*T
      ENDIF
C
      IF (T.GE.6.34D+02 .AND. T.LT.7.40D+02) THEN
	G = 0.13022482D+04 + 0.522110231*T - 5.980360202D-03*T*T
     &    + 6.659625695D-06*T*T*T - 2.473169175D-09*T*T*T*T
      ENDIF
C
      IF (T.GE.7.40D+02 .AND. T.LT.8.60D+02) THEN
	G = - 0.64489093D+05 + 3.095104796D+02*T - 0.547164618*T*T
     &      + 4.255071973D-04*T*T*T - 1.233061536D-07*T*T*T*T
      ENDIF
C
      IF (T.GE.8.60D+02 .AND. T.LT.9.27D+02) THEN
	G = + 0.13015055D+07 - 5.763825D+03*T + 9.571875*T*T
     &      - 7.0625D-03*T*T*T + 1.953125D-06*T*T*T*T
      ENDIF
C
      IF (T.GE.9.27D+02 .AND. T.LT.1.08D+03) THEN
	G = + 0.58257237D+05 - 2.229275965D+02*T + 0.322821329*T*T
     &      - 2.089186484D-04*T*T*T + 5.084325376D-08*T*T*T*T
      ENDIF
C
      IF (T.GE.1.08D+03 .AND. T.LT.1.24D+03) THEN
	G = 0.14043448D+06 - 4.704834516D+02*T + 0.591517384*T*T
     &    - 3.306547594D-04*T*T*T + 6.93139101D-08*T*T*T*T
      ENDIF
C
      IF (T.GE.1.24D+03 .AND. T.LT.1.502D+03) THEN
	G = 0.15744376D+05 - 4.475440789D+01*T + 4.7873355D-02*T*T
     &    - 2.286184211D-05*T*T*T + 4.111842105D-09*T*T*T*T
      ENDIF
C
      IF (T.GE.1.502D+03 .AND. T.LT.1.616D+03) THEN
	G = - 0.58637400D+04 + 1.147433313D+01*T
     &      - 7.5500D-03*T*T + 1.6666666D-06*T*T*T
      ENDIF
C
      IF (T.GE.1.616D+03 .AND. T.LT.1.68D+03) THEN
	G = 0.32972410D+06 - 8.211649405D+02*T + 0.766426516*T*T
     &    - 3.177724674D-04*T*T*T + 4.938936812D-08*T*T*T*T
      ENDIF
C
      IF (T.GE.1.68D+03) THEN
	G = 0.33271186D+07 - 7.657409945D+03*T
     &    + 6.607976863*T*T - 2.534098538D-03*T*T*T
     &    + 3.643939352D-07*T*T*T*T
      ENDIF
C
C Convert units from KCW's cal/mol to J/mol:
      G = G * 8.314D+00 / 1.9858D+00
      GC = - 6.41114D4 + 3.2158D1*T
C Carbon: Kirkaldy and Baganis, Met.Trans 9A 495-501 1978.
C
      RETURN
      END
C**********************************************************************
C
C FUNCTION GIVING THE EQUILIBRIUM MOL.FRAC. CARBON IN ALPHA
C TEMP READ IN AS CENTIGRADE
C BASED ON MY PAPER ON FIRST ORDER QUASICHEMICAL THEROY,METSCI
C
      DOUBLE PRECISION FUNCTION XALPH(T)
      IMPLICIT NONE
      DOUBLE PRECISION T,CTEMP
C
      CTEMP=T/900.0D+00
      XALPH=0.1528D-02-0.8816D-02*CTEMP+0.2450D-01*CTEMP**2
     &-0.2417D-01*CTEMP**3+
     &0.6966D-02*CTEMP**4
C
      RETURN
      END
C**********************************************************************
C
C DAFEG DAFEG DIFFERENTIAL OF LN ACTIVITY OF IRON IN GAMMA
C
      DOUBLE PRECISION FUNCTION DAFEG(XEQ,AJ)
      DOUBLE PRECISION ETEQ,ETEQ2,DEQ,XEQ,AJ
      DEQ=DSQRT(1-2*(1+2*AJ)*XEQ+(1+8*AJ)*XEQ*XEQ)
      ETEQ=5*((1/(XEQ-1))+2/(1-2*XEQ))
      ETEQ2=6*((4*AJ-1-(0.5/DEQ)*(-2-4*AJ+2*XEQ+16*XEQ*AJ))
     &/(1-2*AJ+(4*AJ-1)*XEQ-DEQ))+6*(4*AJ/(2*AJ*(2*XEQ-1)))
      DAFEG=ETEQ+ETEQ2
      RETURN
      END
C
C*****************************************************************
C AFEG AFEG LN ACTIVITY OF IRON IN GAMMA
      DOUBLE PRECISION FUNCTION AFEG(XEQ,AJ)
      DOUBLE PRECISION XEQ,AJ,DEQ,TEQ
      DEQ=DSQRT(1-2*(1+2*AJ)*XEQ+(1+8*AJ)*XEQ*XEQ)
      TEQ=5*DLOG((1-XEQ)/(1-2*XEQ))
      TEQ=TEQ+6*DLOG(((1-2*AJ+(4*AJ-1)*XEQ-DEQ)/(2*AJ*(2*XEQ-1))))
      AFEG=TEQ
      RETURN
      END
C
C************************************************************
C H. K. D. H. Bhadeshia, Diffusivity of carbon in austenite
C Department of Materials Science and Metallurgy
C Pembroke Street, Cambridge CB2 3QZ, U. K.
C 31 March 1987
C
C Program for the calculation of the diffusivity of carbon in austenite
C as a function of carbon concentration and temperature. Also accounts for
C the influence of C, Mn, Si, Ni, Mo, Cr and V on the activity of carbon
C in austenite, and therefore on the diffusivity of carbon.
C
C WDIFF is the carbon--carbon interaction energy in austenite, not
C allowing for the presence of Mo, Cr or V
C
C diffusion coefficient in cm**/s
C 
      SUBROUTINE DIFFUS(DIF,X,T,WDIFF)
      IMPLICIT NONE
	DOUBLE PRECISION DIF,X,T,W,WDIFF,HH,BOLTZ,Z,A5,R
     &, DASH,THETA,ACTIV,DACTIV,SIGMA,PSI,CG,DCG      
C
         PARAMETER (BOLTZ=1.38062D-23,R=8.31432D+00)
         W=WDIFF
         HH=6.6262D-34
         Z=12
         A5=1.0D+00
C         
C     D=DIFFUSIVITY OF CARBON IN AUSTENITE
C     Z=COORDINATION OF INTERSTIAL SITE
C     PSI=COMPOSITION DEPENDENCE OF DIFFUSION COEFFICIENT
C     THETA=NO. C ATOMS/ NO. FE ATOMS
C     ACTIV=ACTIVITY OF CARBON IN AUSTENITE
C     R=GAS CONSTANT
C     X=MOLE FRACTION OF CARBON
C     T=ABSOLUTE TEMPERATURE
C
C     SIGMA=SITE EXCLUSION PROBABLITY
C     W=CARBON CARBON INTERACTION ENERGY IN AUSTENITE
C
      DASH=(BOLTZ*T/HH)*DEXP(-(21230.0D+00/T))*DEXP(-31.84D+00)
      THETA=X/(A5-X)
      ACTIV=CG(X,T,W,R)
      ACTIV=DEXP(ACTIV)
      DACTIV=DCG(X,T,W,R)
      DACTIV=DACTIV*ACTIV
      DACTIV=DACTIV*A5/((A5+THETA)**2)
      SIGMA=A5-DEXP((-(W))/(R*T))
      PSI=ACTIV*(A5+Z*((A5+THETA)/(A5-(A5+Z/2)*THETA+(Z/2)*(A5+Z/2)*
     &(A5-SIGMA)*THETA*THETA)))+(A5+THETA)*DACTIV
      DIF=DASH*PSI
      RETURN
      END

C*********************************************************************
C SUBROUTINE TO CONVERT WT% TO MOLE FRACTION
C                     OR MOLE FRACTION TO WT%.
C N=0 changes from atomic fraction to weight percent
C Any other value of N converts weight to atomic fraction
C 1=C, 2=SI, 3=MN, 4=NI, 5=MO, 6=CR, 7=V, 8=FE
      SUBROUTINE CONV(N,WT,C)
      IMPLICIT NONE
      DOUBLE PRECISION AN(8), WT(8), C(8), A(8)
	DOUBLE PRECISION AT
      INTEGER I,N
C
      AN(1)=12.0115D+00
      AN(2)=28.09D+00
      AN(3)=54.94D+00
      AN(4)=58.71D+00
      AN(6)=95.94D+00
      AN(5)=52.00D+00
      AN(7)=63.55D+00
      AN(8)=55.84D+00
      IF (N .EQ. 0) GOTO 1
      WT(8)=100.0D+00-WT(1)-WT(2)-WT(3)-WT(4)-WT(5)-WT(6)-WT(7)
      AT=0.0D+00
      DO 2 I=1,8
         A(I)=WT(I)/AN(I)
         AT=AT+A(I)
2     CONTINUE
      DO 3 I=1,8
         C(I)=WT(I)/AN(I)/AT
3     CONTINUE
      GOTO 4
1     C(8)=1.0D+00-C(1)-C(2)-C(3)-C(4)-C(5)-C(6)-C(7)
      AT=0.0D+00
      DO 5 I=1,8
         A(I)=C(I)*AN(I)
         AT=AT+A(I)
5     CONTINUE
      DO 6 I=1,8
         WT(I)=C(I)*AN(I)/AT*100.0D+00
6     CONTINUE
4     RETURN
      END
C
C*********************************************************************
C
 	DOUBLE PRECISION FUNCTION RTSAFE(XN,XG,xe,LINEAL,TIME,D)
C		
C	Using a combination of Newton-Raphson and bisection,
C	find the root of a function bracketed between X1 and
C	X2. The root, returned as the function RTSAFE, will
C	be refined until its accuracy is known within +/-
C	XACC. FUNCD is a user-supplied subroutine which returns
C	both the function value and its first derivative.
C
C
C C.Capdevila-Montes. (Cambridge,1997)
C
        IMPLICIT NONE
        DOUBLE PRECISION F,X1,X2,TI,LINEAL,XGAG1,XNORM1,FL,FH,XG,XN
        DOUBLE PRECISION DF,DX,DXOLD,TEMP,XH,XL,D,z2,time,xe,XFEC,Z1
     &,                  C2,XACC,CON2,FLO
	  INTEGER J,MAXIT
	  PARAMETER (MAXIT=100)
C Parameters
        XNORM1=XN
        XGAG1=XG
	  xfec=xe
        LINEAL=LINEAL
        TI=TIME

C define bracketing values and accuracy of end test
        z2=lineal*(1-(xn/xe))
	  z1=lineal*((xg-xn)/(xn+xg))
        c2=(xn/xe)
	  X1=0.0
	  X2=0.99999999d+00*z2
        XACC=1.0D-8
c        write(*,*)x1,x2,lineal,ti,d,xn,xg
        CON2=FLO(Z1,LINEAL,C2)

C maximum allowed no. of iterations
	  CALL FUNCD(X1,FL,DF,XNORM1,XGAG1,xfec,LINEAL,TI,D,CON2)
        CALL FUNCD(X2,FH,DF,XNORM1,XGAG1,xfec,LINEAL,TI,D,CON2)
c        write(*,*)fl,fh,df
        IF ((FL.GT.0..AND.FH.GT.0).OR.(FL.LT.0..AND.FH.LT.0.))THEN
             PAUSE'root must be bracketed in rtsafe'
        ENDIF
C orient the search s.t.  f(XL)<0
       IF (FL .LT. 0) THEN 
		XL=X1
		XH=X2
	 ELSE
		XH=X1
		XL=X2
	 ENDIF
C Initialise the guess for root
	RTSAFE=0.5*(X1+X2)
C Initialise the step size before last
	DXOLD=ABS(X2-X1)
C Initialise the last step
	DX=DXOLD
	CALL FUNCD(RTSAFE,F,DF,XNORM1,XGAG1,xfec,LINEAL,TI,D,con2)
C Loop over allowed iterations
	DO 11 J=1, MAXIT
c	write(*,10) j,rtsafe,dx,xl,xh
	   IF (((RTSAFE-XH)*DF-F)*((RTSAFE-XL)*DF-F) .GE. 0
     &		.OR. ABS(2.0*F) .GT. ABS(DXOLD*DF)) THEN
C Bisect if Newton out of range or not decreasing fast enough
		DXOLD=DX
		DX=0.5*(XH-XL)
		RTSAFE=XL+DX
		IF (XL .EQ. RTSAFE) RETURN
	   ELSE
		DXOLD=DX
		DX=F/DF
		TEMP=RTSAFE
		RTSAFE=RTSAFE-DX
		IF (TEMP .EQ. RTSAFE) RETURN
C Change in root is negligible
	   ENDIF
C Newton step acceptable. Take it.
	   IF (ABS(DX) .LT. XACC) RETURN
C Convergence criterion
	   CALL FUNCD(RTSAFE,F,DF,XNORM1,XGAG1,xfec,LINEAL,TI,D,CON2)
C The one new function evaluation per iteration
	IF (F .LT. 0) THEN
C Maintain the bracket on the root
         XL=RTSAFE
         FL=F
      ELSE
         XH=RTSAFE
         FH=F
      ENDIF
C
11    CONTINUE
	PAUSE 'RTSAFE exceeding maximum iterations'
	RETURN
	END
C
C**********************************************************************
C
	SUBROUTINE FUNCD(X,FUN,DFUN,X1,X2,x3,AGS,TI,D,CON2)
	IMPLICIT NONE
	DOUBLE PRECISION X,X1,X2,x3,FUN,DFUN,AGS,D,TI,F,DF,CON2
	FUN=F(X,X1,X2,x3,AGS,TI,D,CON2)
	DFUN=DF(X,X1,X2,x3,AGS,D)
	RETURN
	END

C**********************************************************************
C FUNCTION FOR NEWTON'S METHOD
C
      DOUBLE PRECISION FUNCTION F(X,X1,X2,x3,AGS,TI,DIFF,CON2)
	IMPLICIT NONE
      DOUBLE PRECISION X,X1,X2,x3,D,AGS,Z1,Z2,B2,TI,DIFF,Q
      DOUBLE PRECISION CON,CON1,CON2,TI1,C2
C
      D=DIFF
      Z1=AGS*((X2-X1)/(X2+X1))
      C2=(X1/X3)
      B2=AGS*C2
      Z2=AGS*(1-C2)
      CON1=(Z1*Z1-4*B2*Z1-2*Z2*Z1)
      CON=CON1+2*(B2*B2)*CON2
      TI1=CON-((X*X)-4*B2*X-2*Z2*X+2*(B2*B2)*DLOG(Z2-X))
      F=TI-((1/(4*D))*TI1)
      Q=(Z2-X)
      IF (Q .LE. 0.0D+00) THEN 
         PRINT *,' **Warning! Boundary conditions failed. '
      END IF
      RETURN
      END
C*********************************************************************
C DIFERENTIAL FUNCTION FOR NEWTON'S METHOD
C 
      DOUBLE PRECISION FUNCTION DF(X,X1,X2,x3,AGS,DIFF)
      IMPLICIT NONE
	DOUBLE PRECISION X,X1,X2,x3,AGS,DIFF,B2,C2,D,Z2,DF1
C
      C2=(X1/X3)
      B2=AGS*C2
      D=DIFF
      Z2=AGS*(1-C2)
      DF1=(B2**2)/(Z2-X)
      DF=(1/(4*D))*(2*X-4*B2-2*Z2-2*DF1)
      RETURN
      END
C*********************************************************************
C FUNCTION OF LN(Z2-Z1) INTERVAL OF POSITION OF THE INTERFACE UNDER
C IMPINGEMENT.
      DOUBLE PRECISION FUNCTION FLO(Y,L,C2)
	IMPLICIT NONE
      DOUBLE PRECISION Y,L,C2,Z2
C
      Z2=L*(1-C2)
      FLO=DLOG(Z2-Y)
      RETURN
      END
C*********************************************************************
C TIME OF GROWING WITH INMPINGMENT.
C Based on Gilmour et al, Met. Trans. 1972, vol. 3, pp.3215.
      DOUBLE PRECISION FUNCTION FIMP(X,X1,X2,x3,AGS,DIFF,CON2)
	IMPLICIT NONE
      DOUBLE PRECISION X1,X2,x3,D,AGS,z1,z2,x,b2,DIFF,Q
      DOUBLE PRECISION C2,CON,CON1,CON2,TI1
      D=DIFF     
      Z1=AGS*((X2-X1)/(X2+X1))
      C2=(X1/X3)
      B2=AGS*C2
      Z2=AGS*(1-C2)
      CON1=(Z1*Z1-4.0D+00*B2*Z1-2.0D+00*Z2*Z1)
      CON=CON1+2.0D+00*(B2*B2)*CON2
      TI1=CON-((X*X)-4.0D+00*B2*X-2.0D+00*Z2*X+2.0D+00*(B2*B2)
     & 	*DLOG(Z2-X))
      FIMP=((1/(4.0D+00*D))*TI1)
      Q=(Z2-X)
      IF (Q .LE. 0.0D+00) THEN 
         PRINT *,' *Warning! Boundary conditions failed. '
      END IF
      RETURN
      END
C**********************************************************************
C
      SUBROUTINE AARG(ARG)
C HELPS AVOID UNDERFLOW ERRORS IN EXPONENTIALS
      IMPLICIT NONE
      DOUBLE PRECISION ARG
      IF(ARG .GT. 173.0D+00) GOTO 1
      GOTO 2
1     ARG=179.0D+00
2     RETURN
      END
C
C********************************************************************
C To calculate the growth kinetics of pearlite
C
       SUBROUTINE PEARL(W1,W2,W3,W4,W5,W6,W7,TI,ISAMPLE,IPARA,
     & PX1GC,PX2GC,PX1CG,PX2CG,PX1GA,PX2GA,PX1AG,PX2AG,VEL1,
     & X1GC,X2GC,X1CG,X2CG,X1GA,X2GA,X1AG,X2AG, 
     & VEL2,SC,S,JPEARL )
C
C PROGRAM TO CALCULATE THE GROWTH RATE OF PEARLITE ASSUMING THE BULK OR
C INTERFACE DIFFUSION CONTROL.
C
C The method applies to steels containing just one substitutional solute
C but all solutes if paraequilibrium
C 
C GAMMA/GAMMA+ALPHA AND GAMMA/GAMMA+THETA INTERFACE COMPOSITIONS ARE
C CALCULATED USING THE NEWTON-RAPHSON METHOD ASSUMING HILLERT-STAFFANSON
C REGULAR SOLUTION MODEL.
C
C Initial guess values are calculated by Kirkaldy's approximate method
C   J. S. Kirkaldy, B. A. Thomson and E. A. Baganis, 
C   "Hardenability Concepts with Applications to Steel"
C   Editors D. V. Doane and J. S. Kirkaldy
C   Published by The Metallurgical Society of the American Inst. 
C   Mining, Metallurgical and Petroleum Engineers, 
C   Warrendale, Pennsylvania, 1978, pp. 83-125.
C
C                                                 1991.5.12 M.TAKAHASHI
C
C Set IPARA=1 if only paraequilibrium rate of growth required
C  otherwise, substitutional solute diffusion with grain boundary 
C  diffusion-controlled growth with IPARA=0
C
      IMPLICIT NONE
      DOUBLE PRECISION X(8),Y(8),C0(8),W0(8),WW(8),K(8),G0(8)
     &,G(8),G2(8),G3(8),G4(8),GM(8),LFX(4,8),AC(8),E(8,8),EA(8,8)
      DOUBLE PRECISION ISAMPLE,IS,W1,W2,W3,W4,W5,W6,W7,DT,TF,TI
     &, KK,YY,CTEMP,T,DG,PX1GC,PX2GC,PX1CG,PX2CG,PX1GA,PX2GA
     &, PX1AG,PX2AG,VEL1,X1GC,X2GC,X1CG,X2CG,X1GA,X2GA,X1AG
     &, X2AG,VEL2,SC,S,GFEC,GFE3C,SFG,LCV,DDG,CC,CCC,D,ACTIV
     &, X1GANP,X2GANP,X1AGNP,X2AGNP,X2TRANS,X1GCNP,X2GCNP,X1CGNP
     &, X2CGNP,VBD,X2AC0,ACG0,VD,T10,T20,DIFF,ENERGY2,W
	INTEGER I,IFLAGC,IFLAGF,IM,IPARA,JPEARL
C
      COMMON/TRANS/C0,W0,K
      COMMON/HILL/G2,LFX,LCV,GM,AC,DDG,GFEC
     &           ,GFE3C,G3,G4,WW,E,EA,G,G0
C
      IS=0
10    IS=IS+1
      IF (IS .GT. ISAMPLE) GOTO 100
C
C Read the concentrations in units of weight percent
      W0(1)=W1
      W0(2)=W2
      W0(3)=W3
      W0(4)=W4
      W0(5)=W5
      W0(6)=W6
      W0(7)=W7
C
C Read the start temperature, temperature interval and finish temperature
C  in degrees centigrade
C   
      DT=10.0D+00
      TF=TI
C
C
C Convert weight percent into mole fraction
      CALL CONV(1,W0,C0)
C
C Calculate carbon-carbon interaction energy
      CALL OMEGA2(W)
C
C
C Compute the mole fraction of iron by difference
      CC=0.0D+00
      DO 11 I=1,7
        CC=CC+C0(I)
11    CONTINUE
      C0(8)=1.0D+00-CC
C
C Calculate the substitutional solute to iron atom ratios
      KK=0.0D+00
      DO 12 I=2,7
        K(I)=C0(I)/C0(8)
        KK=KK+K(I)
        IF (C0(I) .EQ. 0.0D+00) GOTO 12
        IM=I
12    CONTINUE
      YY=0.0D+00
      DO 13 I=2,7
        Y(I)=K(I)/(1.0D+00+KK)
        YY=YY+Y(I)
13    CONTINUE
      Y(8)=1.0D+00-YY
C
C
      CTEMP=TI+DT
20    CTEMP=CTEMP-DT
      T=CTEMP+273.0D+00
      IF (CTEMP .LT. TF) GOTO 100
      IFLAGF=0
      IFLAGC=0
C
C
C THERMODYNAMIC PARAMETERS IN ORDER OF SI,MN,NI,CR,MO,CU
C
C
C*************************
C   The following thermodynamic data are from the following references
C
C   B. Uhrenius, "Hardenability Concepts with Applications to Steel"
C   Editors D. V. Doane and J. S. Kirkaldy
C   Published by The Metallurgical Society of the American Inst. 
C   Mining, Metallurgical and Petroleum Engineers, 
C   Warrendale, Pennsylvania, 1978, pp. 28-82

C   J. S. Kirkaldy, B. A. Thomson and E. A. Baganis, 
C   "Hardenability Concepts with Applications to Steel"
C   Editors D. V. Doane and J. S. Kirkaldy
C   Published by The Metallurgical Society of the American Inst. 
C   Mining, Metallurgical and Petroleum Engineers, 
C   Warrendale, Pennsylvania, 1978, pp. 83-125.
C*************************
C-----------------------------------------------------------
C THERMODYNAMIC PARAMETERS
C IN ORDER OF C, SI,MN,NI,CR,MO,CU.
C
      DG=46150.0D+00/3.0-19.205D+00*T/3.0
      GFE3C=1.332D+04-64.718*T+7.481*T*DLOG(T)-DG
      SFG=-11.906D-04*T+8.272D-06*T*T-15.079D-09*T*T*T
     &     +12.857D-12*T*T*T*T
      G0(1)=-65562.0D+00+32.949*T
      G0(3)=-20520.0D+00+4.086D+00*T+1500.0D+00*SFG
      G0(5)=-1534.0D+00-19.472*T+2.749*T*DLOG(T)
      G(2)=28535.0D+00-DG
      G(3)=-14263.0D+00+10.0D+00*T-DG
C     G(3)=-13532.0D+00-DG
      G(4)=20338.0D+00-2.368D+00*T-DG
      G(5)=-24418.0D+00+16.61D+00*T-2.749D+00*T*DLOG(T)-DG
      G(6)=-19644.0D+00-0.628*T-DG
      G(7)=28535.0D+00-DG
C
C Carbon-carbon and carbon-substitutional solute interaction parameters
C  as a function of temperature, p.87 of Kirkaldy et al.
C
C     E(1,1)=4.7859D+00+5066.0D+00/T
      E(1,1)=8910.0D+00/T
C     E(1,2)=4.84D+00-7370.0D+00/T
      E(1,2)=14795.0D+00/T
      E(1,3)=-4811.0D+00/T
      E(1,4)=-2.2D+00+7600.0D+00/T
      E(1,5)=24.4-38400.0D+00/T
      E(1,6)=3.855D+00-17870.0D+00/T
      E(1,7)=4200.0D+00/T
C
      E(2,2)=26048.0D+00/T
      E(3,3)=2.406D+00-175.6D+00/T
C     E(3,3)=0.2D+00
      E(4,4)=-721.7D+00/T
      E(5,5)=7.655D+00-3154.0D+00/T-0.661D+00*DLOG(T)
      E(6,6)=-2330.0D+00/T
      E(7,7)=-0.161D+00-7834.0D+00/T
C
      EA(1,1)=1.3D+00
      EA(2,2)=-13.31D+00+44088.0/T
      EA(3,3)=3.082D+00-4679.0/T+1509.8*SFG/T
      EA(4,4)=2.041D+00-2478.0/T+385.5*SFG/T
      EA(5,5)=2.819D+00-6039.0/T
      EA(6,6)=-0.219D+00-4772.0/T+402.6*SFG/T
      EA(7,7)=0.634D+00-11270.0/T+1006.5*SFG/T
C
      WW(2)=0.0D+00
      WW(3)=8351.0D+00-15.188D+00*T
      WW(4)=0.0D+00
      WW(5)=1791.0D+00
      WW(6)=0.0D+00
      WW(7)=0.0D+00
C---------------------------------------------------------
      G2(2)=123000D+00
      G2(3)=-48500D+00
      G2(4)=46000D+00
      G2(5)=-251160D+00+118.0D+00*T
      G2(6)=-267200D+00
      G2(7)=-46000.0D+00+55.0D+00*T
      G3(2)=404180.0D+00
      G3(3)=-145500.0D+00
      G3(4)=138000.0D+00
      G3(5)=-153640.0D+00+38.860*T
      G3(6)=-267200.0D+00
      G3(7)=-46000.0D+00+55.0*T
      G4(1)=-65563.0D+00+23.815*T
      G4(2)=-64818.0D+00+38.543*T
      G4(3)=-1800.0D+00+1.276*T
      G4(4)=-5650.0D+00-3.35*T
      G4(5)=10460.0D+00+0.628*T
      G4(6)=10460.0D+00+0.628*T
      G4(7)=-6276.0D+00+3.347*T
      G4(8)=(-1.0D+00)*ENERGY2(T,T10,T20)
C
      LFX(1,2)=-108280.0D+00
      LFX(1,3)=730.0D+00-10.0D+00*T
      LFX(1,4)=-14600.0D+00
      LFX(1,5)=13110.0D+00-31.82D+00*T+2.748D+00*T*DLOG(T)
      LFX(1,6)=9686.0D+00
      LFX(1,7)=49752.0D+00-9.431D+00*T
      LFX(2,2)=0.0D+00
      LFX(2,3)=0.0D+00
      LFX(2,4)=8800.0D+00
      LFX(2,5)=0.0D+00
      LFX(2,6)=0.0D+00
      LFX(2,7)=-8594.0D+00+5.05D+00*T
      LFX(3,2)=-177100.0D+00+30.02*T
      LFX(3,3)=19450.0D+00-12.81*T-1500.0*SFG
      LFX(3,4)=-7300.0D+00-8.73*T-383.0*SFG
      LFX(3,5)=25104.0D+00-11.720*T
      LFX(3,6)=19836.0D+00+0.912*T-400.0*SFG
      LFX(3,7)=46848.0D+00-2.636*T-1000.0*SFG
      LFX(4,2)=-3085.0D+00+12.65*T
      LFX(4,3)=0.0D+00
      LFX(4,4)=8800.0D+00
      LFX(4,5)=0.0D+00
      LFX(4,6)=0.0D+00
      LFX(4,7)=0.0D+00
      LCV=-21058.0D+00-11.581D+00*T
C     GM(2)=28535.0D+00
      GM(2)=0.0D+00
      GM(3)=-13532.0D+00
      GM(4)=14540.0D+00-2.367D+00*T
      GM(5)=-850.0D+00-14.58D+00*T
      GM(6)=502.0D+00
      GM(7)=28535.0D+00
      AC(2)=0.0D+00
      AC(3)=8350.0D+00-15.2D+00*T
      AC(4)=0.0D+00
      AC(5)=1790.0D+00
      AC(6)=0.0D+00
      AC(7)=0.0D+00
      GFEC=1.332D+04-64.718*T+7.481*T*DLOG(T)-DG
      DDG=46150.0D+00-19.221D+00*T
C------------------------------------------------------------
C
C CALCULATION OF PARAEQUILIBRIUM COMPOSITIONS (GAMMA/GAMMA+ALPHA)
C
C
        CALL PEQGA0(T,X,Y,IM)
C
C Mole fraction of carbon in austenite in paraequilibrium with ferrite
        PX1GA=X(1)
C Mole fraction of substitutional solute in austenite in paraeq. with ferrite
        PX2GA=X(IM)
C Mole fraction of carbon in ferrite in paraequilibrium with austenite
        PX1AG=Y(1)
C Mole fracation of substitutional solute in ferrite in paraeq. with ferrite
        PX2AG=Y(IM)
C
C
C CALCULATION OF PARAEQUILIBRIUM COMPOSITIONS (GAMMA/GAMMA+THETA)
C
C
        CALL PEQGC0(T,X,Y,IM)
C
C Mole fraction of carbon in austenite in paraequilibrium with cementite
        PX1GC=X(1)
C Mole fraction of susbtitutional solute in austenite in paraeq. with cementite
        PX2GC=X(IM)
C Mole fraction carbon in cementite paraequilibrium with austenite
        PX1CG=0.25D+00
C Mole fraction substitutional solute in cementite in paraeq. with austenite
        PX2CG=Y(IM)
C
C
C CALCULATION OF VOLUME DIFFUSION CONTROLLED GROWTH RATE OF PEARLITE
C SC and S are the empirical critical and actual interlamellar spacings
C  This is in units of cm. Error in Manabu's original program, in which
C  the temperature should have been in Kelvin in SC corrected 13/5/95.
C  The 1000 in the equation for SC refers to the eutectoid temperature in
C  Kelvin for plain carbon steel.
C The equation for S is derived from data in Manabu's thesis, including
C  only the effects of Mn and temperature (centigrade)
C
C
      SC=2.0*600.0*1000.0/6.09D09/(1000.0D+00-(CTEMP+273.15))
      S=1.0D-04/(127.351D+00-0.17368D+00*CTEMP-4.9195*W0(3)
     &           +1.7868*W0(5))
C
C
      CCC=(PX1GA+PX1GC)/2.0
      D=DIFF(T,CCC,CCC*1.001,W)
C
        IF(JPEARL .EQ. 1) THEN
           VEL1=VD(PX1GA,PX1GC,PX1AG,SC,D,S)
        ELSE
           VEL1=0.0D+00
        ENDIF
C
C
C CALCULATION OF THE INTERFACE COMPOSITIONS (GAMMA/GAMMA+ALPHA)
C IDENTICAL ISOACTIVITY OF CARBON IN GAMMA IS ASSUMED
C
C CALCULATION OF ACTIVITY OF CARBON IN GAMMA
C
       ACTIV=ACG0(T,C0(1),C0(IM),E(1,1),E(1,IM))
C
C FIRST GUESS VALUES FOR THE INTERFACE 
C
      CALL NPGA0(T,X,Y,IM)
        X1GANP=X(1)
        X2GANP=X(IM)
        X1AGNP=Y(1)
        X2AGNP=Y(IM)
C
      X2TRANS=X2AC0(T,X1GANP,ACTIV,E(1,1),E(1,IM))
C      write(6,*) 'X2GANP,X2TRANS = ',X2GANP,X2TRANS
      IF (X2GANP .LT. X2TRANS) THEN
C       WRITE(6,*) 'ENTERING NPLE REGIME FOR FERRITE.....'
        IFLAGF=1
        X1GA=X1GANP
        X2GA=X2GANP
        X1AG=X1AGNP
        X2AG=X2AGNP
      ELSE
       CALL EQGA0(T,X,Y,ACTIV,IM)
        X1GA=X(1)
        X2GA=X(IM)
        X1AG=Y(1)
        X2AG=Y(IM)
      ENDIF
C
C CALCULATION OF THE INTERFACE COMPOSITIONS (GAMMA/GAMMA+THETA)
C IDENTICAL ISOACTIVITY OF CARBON IN GAMMAAT THE INTERFACES IS ASSUMED
C
C FIRST GUESS VALUES FOR THE INTERFACE
C 
      CALL NPGC0(T,X,Y,IM)
        X1GCNP=X(1)
        X2GCNP=X(IM)
        X1CGNP=0.25D+00
        X2CGNP=Y(IM)
C
      X2TRANS=X2AC0(T,X1GCNP,ACTIV,E(1,1),E(1,IM))
      IF(X2GCNP .GT. X2TRANS) THEN
C        WRITE(6,*) 'ENTERING NPLE REGIME FOR CEMENTITE......'
        IFLAGC=1
        X1GC=X1GCNP
        X2GC=X2GCNP
        X1CG=X1CGNP
        X2CG=X2CGNP
      ELSE
       CALL EQGC0(T,X,Y,ACTIV,IM)
        X1GC=X(1)
        X2GC=X(IM)
        X1CG=0.25D+00
        X2CG=Y(IM)
      ENDIF
C
C
C CALCULATION OF BOUNDARY DIFFUSION CONTROLLED GROWTH RATE OF PEARLITE
C
         IF(JPEARL .EQ. 1) THEN
           VEL2=VBD(T,X2GA,X2GC,SC,C0(IM),S,IM)
         ELSE
           VEL2=0.0D+00
         ENDIF
C
50    CONTINUE
C
      GOTO 20
C
100    RETURN
       END
C
C
C***********************************************************************
C
C SUBROUTINE TO CALCULATE INTERFACE COMPOSITIONS (GAMMA/GAMMA+ALPHA)
C  Kirkaldy's approximate method is used to obtain the first guess value
C  NPLE calculation
C
      SUBROUTINE NPGA0(T,X,Y,IM)
      IMPLICIT NONE
      DOUBLE PRECISION X(8),Y(8),C0(8),W0(8),WW(8),K(8)
     &,XX(3),G(8),G2(8),G3(8),G4(8),GM(8),G0(8)
     &,LFX(4,8),AC(8),E(8,8),EA(8,8),FUN(3)
	DOUBLE PRECISION R,XGA,ACC,DX1,X1,X2,A1,A2,T,Y1,Y2
     &, X0,Y0,FFA,FFG,DUMMY,LCV,GFEC,GFE3C,DDG
     	INTEGER I,IM,ITER
C
      COMMON/TRANS/C0,W0,K
      COMMON/HILL/G2,LFX,LCV,GM,AC,DDG,GFEC,
     &            GFE3C,G3,G4,WW,E,EA,G,G0
C
C      write(6,*) 'IM=',IM
      R=8.314D+00
      XGA=C0(1)+0.01D+00
      ITER=0
      ACC=1.0D-05
      DX1=1.0D-10
200   XX(1)=XGA-DX1
      XX(2)=XGA+DX1
      XX(3)=XGA
	ITER=ITER+1
      IF(ITER .GT. 10) THEN
         ACC=ACC*10
         ITER=0
      ENDIF
      DO 220 I=1,3
        X1=XX(I)
C
      A1=DEXP(G0(1)/R/T+E(1,1)*X1)/
     &       (1.0D+00+EA(1,1)*X1*DEXP(G0(1)/R/T))
      A2=DEXP(G0(IM)/R/T+E(1,IM)*X1)/
     &       (1.0D+00+E(1,IM)*X1*DEXP(G0(1)/R/T))
C      write(6,*) 'A1,A2=',A1,A2
      Y1=A1*X1
      Y2=C0(IM)
      X2=Y2/A2
C
C      write(6,*) 'X2AC=',X2
      X0=1.0D+00-X1-X2
	Y0=1.0D+00-Y1-Y2
C
      FFG=DLOG(X0)-1.0/2.0*E(1,1)*X1*X1-1.0/2.0*E(IM,IM)*X2*X2
     &    -E(1,IM)*X1*X2
      FFA=DLOG(Y0)-1.0/2.0*EA(1,1)*Y1*Y1-1.0/2.0*EA(IM,IM)*Y2*Y2
     &    -E(1,IM)*Y1*Y2
C    
      FUN(I)=G4(8)/R/T+FFG-FFA
C
220   CONTINUE    
      IF (ABS(FUN(3)) .LT. ACC) GOTO 100
      DUMMY=XGA-2.0*DX1*FUN(3)/(FUN(2)-FUN(1))
      IF (DUMMY .LT. 0.0D+00) THEN
	   DUMMY=XGA/2.0D+00
      ENDIF
C
      IF (DUMMY .GT. 0.25D+00) THEN
        DUMMY=(0.25D+00+XGA)/2.0
      ENDIF
C
      XGA=DUMMY
C      write(6,*) 'FUN3,X1=',FUN(3),XGA
      GOTO 200
C
100   X(1)=X1
      X(IM)=X2
      Y(1)=Y1
      Y(IM)=Y2
C
      RETURN
      END
C
C***********************************************************************
C
C SUBROUTINE TO CALCULATE INTERFACE COMPOSITIONS (GAMMA/GAMMA+ALPHA)
C  Kirkaldy's approximate method is used to obtain the first guess value
C
      SUBROUTINE EQGA0(T,X,Y,ACTIV,IM)
      IMPLICIT NONE
      DOUBLE PRECISION X(8),Y(8),C0(8),W0(8),WW(8),K(8)
     &,XX(3),G(8),G2(8),G3(8),G4(8),GM(8),G0(8)
     &,LFX(4,8),AC(8),E(8,8),EA(8,8),FUN(3)
	DOUBLE PRECISION R,DX1,XGA,X1,X2,A1,A2,Y1,Y2,T
     &, X0,Y0,FFA,FFG,DUMMY,ACTIV,LCV,GFEC,GFE3C,X2AC0
     &, DDG

      INTEGER I,IM
C
      COMMON/TRANS/C0,W0,K
      COMMON/HILL/G2,LFX,LCV,GM,AC,DDG,GFEC,
     &            GFE3C,G3,G4,WW,E,EA,G,G0
C
      R=8.314D+00
      XGA=C0(1)*1.1D+00
      DX1=1.0D-06
200   XX(1)=XGA-DX1
      XX(2)=XGA+DX1
      XX(3)=XGA
      DO 220 I=1,3
        X1=XX(I)
C
      A1=DEXP(G0(1)/R/T+E(1,1)*X1)/
     &       (1.0D+00+EA(1,1)*X1*DEXP(G0(1)/R/T))
      A2=DEXP(G0(IM)/R/T+E(1,IM)*X1)/
     &       (1.0D+00+E(1,IM)*X1*DEXP(G0(1)/R/T))
      Y1=A1*X1
C
      X2=X2AC0(T,X1,ACTIV,E(1,1),E(1,IM))
      Y2=A2*X2
      X0=1.0D+00-X1-X2
      Y0=1.0D+00-Y1-Y2
C
        FFG=DLOG(X0)-1.0/2.0*E(1,1)*X1*X1-1.0/2.0*E(IM,IM)*X2*X2
     &      -E(1,IM)*X1*X2
        FFA=DLOG(Y0)-1.0/2.0*EA(1,1)*Y1*Y1-1.0/2.0*EA(IM,IM)*Y2*Y2
     &      -E(1,IM)*Y1*Y2
C    
        FUN(I)=G4(8)/R/T+FFG-FFA
220   CONTINUE    
      IF (ABS(FUN(3)) .LT. 1.0D-06) GOTO 100
        DUMMY=XGA-2.0*DX1*FUN(3)/(FUN(2)-FUN(1))
        IF (DUMMY .LT. 0.0D+00) THEN
          DUMMY=XGA/2.0D+00
        ENDIF
        IF (DUMMY .GT. 1.0D+00) THEN
          DUMMY=(1.0D+00+XGA)/2.0
        ENDIF
        XGA=DUMMY
        GOTO 200
100   X(1)=X1
      X(IM)=X2
      Y(1)=Y1
      Y(IM)=Y2
C
      RETURN
      END
C
C***********************************************************************
C
C SUBROUTINE TO CALCULATE INTERFACE COMPOSITIONS (GAMMA/GAMMA+ALPHA)
C  Kirkaldy's approximate method is used to obtain the first guess value
C                  -paraequilibrium-
C
      SUBROUTINE PEQGA0(T,X,Y,IM)
      IMPLICIT NONE
      DOUBLE PRECISION X(8),Y(8),C0(8),W0(8),WW(8)
     &,K(8),XXX(3),FUN1(3),G(8),G2(8),G3(8),G4(8),GM(8)
     &,LFX(4,8),AC(8),E(8,8),EA(8,8),G0(8)
	DOUBLE PRECISION R,T,DX1,X1,X2,Y1,Y2,X0,Y0,A1
     &, FFA,FFG,F2A,F2G,LCV,GFEC,GFE3C,DUMMY1,DDG
      INTEGER I,IM
C
      COMMON/TRANS/C0,W0,K
      COMMON/HILL/G2,LFX,LCV,GM,AC,DDG,GFEC
     &           ,GFE3C,G3,G4,WW,E,EA,G,G0
C
      R=8.314D+00
C
      X1=0.01D+00
      DX1=1.0D-06
200   XXX(1)=X1-DX1
      XXX(2)=X1+DX1
      XXX(3)=X1
      DO 220 I=1,3
        X1=XXX(I)
C
      A1=DEXP(G4(1)/R/T+E(1,1)*X1)/
     &       (1.0D+00+EA(1,1)*X1*DEXP(G4(1)/R/T))
      Y1=A1*X1
      X2=(1.0D+00-X1)*K(IM)/(1.0D+00+K(IM))
      Y2=(1.0D+00-Y1)*K(IM)/(1.0D+00+K(IM))
      X0=1.0D+00-X1-X2
      Y0=1.0D+00-Y1-Y2
C
        FFG=R*T*DLOG(X0)-R*T/2.0*E(1,1)*X1*X1
     &      -R*T/2.0*E(IM,IM)*X2*X2
     &      -R*T*E(1,IM)*X1*X2
        FFA=R*T*DLOG(Y0)-R*T/2.0*EA(1,1)*Y1*Y1
     &      -R*T/2.0*EA(IM,IM)*Y2*Y2
     &      -R*T*E(1,IM)*Y1*Y2
        F2G=R*T*DLOG(X2)+R*T*(E(1,IM)*X1+E(IM,IM)*X2)
        F2A=R*T*DLOG(Y2)+R*T*(E(1,IM)*Y1+EA(IM,IM)*Y2)
      FUN1(I)=X0*(G4(8)+FFG-FFA)+X2*(G4(IM)+F2G-F2A)
C      
220   CONTINUE
      IF (FUN1(3) .LT. 1.0D-06) GOTO 300
        DUMMY1=X1-2.0*DX1*FUN1(3)/(FUN1(2)-FUN1(1))
        IF (DUMMY1 .LT. 0.0D+00) THEN
          DUMMY1=X1/2.0D+00
        ENDIF
        X1=DUMMY1
        GOTO 200
300   X(1)=X1
      X(IM)=X2
      Y(1)=Y1
      Y(IM)=Y2
C
      RETURN
      END
C
C***********************************************************************
C
      DOUBLE PRECISION FUNCTION ACG0(T,X1,X2,E11,E12)
      IMPLICIT NONE
	DOUBLE PRECISION T,X1,X2,E11,E12
C
      ACG0=DEXP(DLOG(X1)+E11*X1+E12*X2)
C
      RETURN
      END
C
C***********************************************************************
C
C FUNCTION TO CALCULATE BOUNDARY DIFFUSION CONTROLLED GROWTH RATE
C OF PEARLITE
C
      DOUBLE PRECISION FUNCTION VBD(T,XGA,XGC,SC,XBAR,S,IM)
      IMPLICIT NONE
      DOUBLE PRECISION CK(8),Q(8)
	DOUBLE PRECISION T,XGA,XGC,SC,XBAR,S,R,SA,SCEM,KDD
	INTEGER IM
C
      CK(3)=1.126D-08
      CK(5)=2.455D-09
      Q(3)=141178.0D+00
      Q(5)=138516.0D+00
      R=8.314D+00
      SA=S*7.0D+00/8.0D+00
      SCEM=S/8.0D+00
      KDD=CK(IM)*DEXP(-Q(IM)/R/T)
      VBD=12.0D+00*KDD/SA/SCEM*(XGA-XGC)/
     &                       XBAR*(1.0D+00-SC/S)
C
      RETURN
      END
C
C*********************************************************************
C
      SUBROUTINE OMEGA2(W)
C SUBROUTINE TO CALCULATE THE CARBON CARBON INTERACTION ENERGY IN
C AUSTENITE, AS A FUNCTION OF ALLOY COMPOSITION.  BASED ON .MUCG18
C THE ANSWER IS IN JOULES PER MOL.   **7 OCTOBER 1981**
      IMPLICIT NONE
      DOUBLE PRECISION C(8),P(8),Y(8),C0(8),W0(8),K(8)
	DOUBLE PRECISION W,B1,B2,B3,T10,T20,XONE
      INTEGER I,U
C	
      COMMON/TRANS/C0,W0,K
C
      DO 1 I=1,8
        C(I)=W0(I)
1     CONTINUE
      B3=0.0D+00
      C(8)=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)
      C(8)=100.0D+00-C(8)
      C(8)=C(8)/55.84D+00
      C(1)=C(1)/12.0115D+00
      C(2)=C(2)/28.09D+00
      C(3)=C(3)/54.94D+00
      C(4)=C(4)/58.71D+00
      C(5)=C(5)/95.94D+00
      C(6)=C(6)/52.0D+00
      C(7)=C(7)/50.94D+00
      B1=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)+C(8)
      DO 107 U=2,7
      Y(U)=C(U)/C(8)
107   CONTINUE
      DO 106 U=1,8
      C(U)=C(U)/B1
106   CONTINUE
      XONE=C(1)
C      XONE=DINT(10000.0D+00*XONE)
C      XONE=XONE/10000
      B2=0.0D+00
      T10=Y(2)*(-3)+Y(3)*2+Y(4)*12+Y(5)*(-9)+Y(6)*(-1)+Y(7)*(-12)
      T20=-3*Y(2)-37.5*Y(3)-6*Y(4)-26*Y(5)-19*Y(6)-44*Y(7)
      P(2)=2013.0341+763.8167*C(2)+45802.87*C(2)**2-280061.63*C(2)**3
     &+3.864D+06*C(2)**4-2.4233D+07*C(2)**5+6.9547D+07*C(2)**6
      P(3)=2012.067-1764.095*C(3)+6287.52*C(3)**2-21647.96*C(3)**3-
     &2.0119D+06*C(3)**4+3.1716D+07*C(3)**5-1.3885D+08*C(3)**6
      P(4)=2006.8017+2330.2424*C(4)-54915.32*C(4)**2+1.6216D+06*C(4)**3
     &-2.4968D+07*C(4)**4+1.8838D+08*C(4)**5-5.5531D+08*C(4)**6
      P(5)=2006.834-2997.314*C(5)-37906.61*C(5)**2+1.0328D+06*C(5)**3
     &-1.3306D+07*C(5)**4+8.411D+07*C(5)**5-2.0826D+08*C(5)**6
      P(6)=2012.367-9224.2655*C(6)+33657.8*C(6)**2-566827.83*C(6)**3
     &+8.5676D+06*C(6)**4-6.7482D+07*C(6)**5 +2.0837D+08*C(6)**6
      P(7)=2011.9996-6247.9118*C(7)+5411.7566*C(7)**2
     &+250118.1085*C(7)**3-4.1676D+06*C(7)**4
      DO 108 U=2,7
      B3=B3+P(U)*Y(U)
      B2=B2+Y(U)
108   CONTINUE
      IF (B2 .EQ. 0.0D+00) GOTO 455
      W=(B3/B2)*4.187
      GOTO 456
455   W=8054.0
456   RETURN
      END
C
C
C **********************************************************************
C
C  SUBROUTINE TO CALCULATE THE PARAEQUILIBRIUM INTERFACE CHEMISTRY
C
      SUBROUTINE PEQGC0(T,X,Y,IM)
      IMPLICIT NONE
      DOUBLE PRECISION X(8),Y(8),C0(8),W0(8),WW(8)
     &,K(8),X1(3),FUN(3),G(8),G2(8),G3(8),G4(8),GM(8)
     &,LFX(4,8),AC(8),E(8,8),EA(8,8),G0(8)
	DOUBLE PRECISION R,XGA,DUMMY,XX,KK,DENO,MC,MFE,MFE3C
     &, MX,MM3C,T,GFEC,GFE3C,DDG,LCV
      INTEGER I,IM
C
      COMMON/TRANS/C0,W0,K
      COMMON/HILL/G2,LFX,LCV,GM,AC,DDG,GFEC
     &           ,GFE3C,G3,G4,WW,E,EA,G,G0
      R=8.314D+00
      XGA=0.005D+00
C
C
      DUMMY=1.0D-08
100   X1(1)=XGA-DUMMY
      X1(2)=XGA+DUMMY
      X1(3)=XGA
      DO 10 I=1,3
       XX=0.0D+00
       FUN(I)=0.0D+00
       KK=K(IM)
       Y(IM)=KK/(1+KK)
       X(IM)=(1-X1(I))*Y(IM)
       X(1)=X1(I)
       X(8)=1.0D+00-X(1)-X(IM)
       Y(8)=1.0D+00-Y(IM)
C
       DENO=1.0D+00
C
      MFE3C=R*T*DLOG(Y(8))
      MFE=R*T*DLOG(X(8)/DENO)
     &         -R*T/2.0*E(1,1)*X(1)*X(1)/DENO/DENO
      MC=R*T/3.0*DLOG(X(1)/DENO)
     &         +R*T/3.0*E(1,1)*X(1)/DENO
      MFE3C=MFE3C+(1-Y(8))*WW(IM)*Y(IM)
      MFE=MFE-R*T/2.0*E(IM,IM)*X(IM)*X(IM)/DENO/DENO
      MFE=MFE-R*T*X(1)*E(1,IM)*X(IM)/DENO/DENO
      MC=MC+R*T/3.0*E(1,IM)*X(IM)/DENO
      MX= R*T*DLOG(X(IM)/DENO)
     &         +R*T*(E(1,IM)*X(1)+E(IM,IM)*X(IM))/DENO
      MM3C= R*T*DLOG(Y(IM))
     &           +Y(8)*WW(IM)-Y(8)*WW(IM)*Y(IM)
C
      FUN(I)=X(8)*(GFE3C+MFE3C-MFE-MC)
      FUN(I)=FUN(I)+X(IM)*(G(IM)+MM3C-MX-MC)
C
C     WRITE(6,*) IM,E(1,1),E(1,IM),E(IM,IM)
C
10    CONTINUE
      IF(ABS(FUN(3)) .LT. 1.0D-6) GOTO 20
      XGA=X1(3)-2.0*DUMMY*FUN(3)/(FUN(2)-FUN(1))
      GOTO 100
20    XGA=X1(3)
      RETURN
      END
C
C****************************************************************
C SUBROUTINE TO CALCULATE THE EQUILIBRIUM INTERFACE CHEMISTRY
C
      SUBROUTINE NPGC0(T,X,Y,IM)
      IMPLICIT NONE
      DOUBLE PRECISION X(8),Y(8),C0(8),W0(8),WW(8)
     &,K(8),XX(3),FUN(3),G0(8)
     &,G(8),G2(8),G3(8),G4(8),GM(8),MX(8),MM3C(8)
     &,LFX(4,8),AC(8),E(8,8),EA(8,8)
	DOUBLE PRECISION R,XGC,DUMMY,X1,X2,Y2,X8,Y8
     &, B,MFE,MFE3C,T,MC,GFEC,GFE3C,DUMMY1,LCV,DDG
      INTEGER I,IM
C
      COMMON/TRANS/C0,W0,K
      COMMON/HILL/G2,LFX,LCV,GM,AC,DDG,GFEC
     &           ,GFE3C,G3,G4,WW,E,EA,G,G0
C
      PARAMETER (R=8.314D+00)
C
      XGC=C0(IM)*0.99D+00
      DUMMY=1.0D-6
100   XX(1)=XGC-DUMMY
      XX(2)=XGC+DUMMY
      XX(3)=XGC
      DO 10 I=1,3
       X1=XX(I)
       FUN(I)=0.0D+00
       B=DEXP((GFE3C-G(IM)-WW(IM))/R/T+E(1,IM)*X1)
       Y2=C0(IM)*4.0/3.0
       X2=Y2/B
       X8=1.0D+00-X1-X2
       Y8=1.0D+00-Y2
      IF (X(8) .LT. 0.0D+00) THEN
         WRITE(6,*) 'X8,X1=',X8,X1
      ENDIF
C
      MFE3C=R*T*DLOG(Y8)
      MFE=R*T*DLOG(X8)-R*T/2.0*E(1,1)*X1*X1
      MC=R*T/3.0*DLOG(X1)+R*T/3.0*E(1,1)*X1
      MFE3C=MFE3C+(1-Y8)*WW(IM)*Y2
      MFE=MFE-R*T/2.0*E(IM,IM)*X2*X2-R*T*X1*E(1,IM)*X2
      MC=MC+R*T/3.0*E(1,IM)*X2
      MX(IM)= R*T*DLOG(X2)
     &         +R*T*(E(1,IM)*X1+E(IM,IM)*X2)
      MM3C(IM)= R*T*DLOG(Y2)
     &           +Y8*WW(IM)
      MM3C(IM)=MM3C(IM)-Y8*WW(IM)*Y2
C
      FUN(I)=GFE3C+MFE3C-MFE-MC
C
10    CONTINUE
      IF(ABS(FUN(3)) .LT. 1.0D-02) GOTO 20
        DUMMY1=XGC-2.0*DUMMY*FUN(3)/(FUN(2)-FUN(1))
        IF (DUMMY1 .LT. 0.0D+00) THEN
         DUMMY1=XGC/2.0D+00
        ENDIF
        XGC=DUMMY1
      GOTO 100
20     X(1)=X1
       X(IM)=X2
       Y(1)=0.25D+00
       Y(IM)=3.0D+00/4.0D+00*Y2
      RETURN
      END
C
C****************************************************************
C SUBROUTINE TO CALCULATE THE EQUILIBRIUM INTERFACE CHEMISTRY
C
      SUBROUTINE EQGC0(T,X,Y,ACTIV,IM)
      IMPLICIT NONE
      DOUBLE PRECISION X(8),Y(8),C0(8),W0(8),WW(8)
     &,K(8),XX(3),FUN(3),G0(8)
     &,G(8),G2(8),G3(8),G4(8),GM(8),MX(8),MM3C(8)
     &,LFX(4,8),AC(8),E(8,8),EA(8,8)
	DOUBLE PRECISION R,T,XGC,X1,X2,ACTIV,B,DUMMY,Y2
     &, X8,Y8,MFE,MFE3C,GFEC,GFE3C,MC,DUMMY1,LCV
     &, X2AC0,DDG
      INTEGER I,IM
C
      COMMON/TRANS/C0,W0,K
      COMMON/HILL/G2,LFX,LCV,GM,AC,DDG,GFEC
     &           ,GFE3C,G3,G4,WW,E,EA,G,G0
C
      PARAMETER (R=8.314D+00)
C
      XGC=C0(IM)*0.99D+00
7     X2=X2AC0(T,XGC,ACTIV,E(1,1),E(1,IM))    
C7     X2=X2AC(T,XGC,ACTIV,LCV,G2(IM))
      IF (X2 .LT. 0.0D+00) THEN
        XGC=(C0(1)+XGC)/2.0
        GOTO 7
      ENDIF    
      DUMMY=1.0D-6
100   XX(1)=XGC-DUMMY
      XX(2)=XGC+DUMMY
      XX(3)=XGC
      DO 10 I=1,3
       X1=XX(I)
       FUN(I)=0.0D+00
       B=DEXP((GFE3C-G(IM)-WW(IM))/R/T+E(1,IM)*X1)
C       X2=C0(IM)*(4.0*X1-1.0D+00)/
C     &    (3.0*B*(X1-C0(1))+4.0*C0(1)-1.0D+00)
C        X2=X2AC(T,X1,ACTIV,LCV,G2(IM))
       X2=X2AC0(T,X1,ACTIV,E(1,1),E(1,IM))
       Y2=B*X2
       X8=1.0D+00-X1-X2
       Y8=1.0D+00-Y2
      IF (X(8) .LT. 0.0D+00) THEN
         WRITE(6,*) 'X8,X1=',X8,X1
      ENDIF
C
C
      MFE3C=R*T*DLOG(Y8)
      MFE=R*T*DLOG(X8)-R*T/2.0*E(1,1)*X1**2
      MC=R*T/3.0*DLOG(X1)+R*T/3.0*E(1,1)*X1
      MFE3C=MFE3C+(1-Y8)*WW(IM)*Y2
      MFE=MFE-R*T/2.0*E(IM,IM)*X2*X2-R*T*X1*E(1,IM)*X2
      MC=MC+R*T/3.0*E(1,IM)*X2
      MX(IM)= R*T*DLOG(X2)
     &         +R*T*(E(1,IM)*X1+E(IM,IM)*X2)
      MM3C(IM)= R*T*DLOG(Y2)
     &           +Y8*WW(IM)
      MM3C(IM)=MM3C(IM)-Y8*WW(IM)*Y2
C
      FUN(I)=GFE3C+MFE3C-MFE-MC
C
10    CONTINUE
      IF(ABS(FUN(3)) .LT. 1.0D-02) GOTO 20
        DUMMY1=XGC-2.0*DUMMY*FUN(3)/(FUN(2)-FUN(1))
        IF (DUMMY1 .LT. 0.0D+00) THEN
         DUMMY1=XGC/2.0D+00
        ENDIF
        XGC=DUMMY1
      GOTO 100
20     X(1)=X1
       X(IM)=X2
       Y(1)=0.25D+00
       Y(IM)=3.0D+00/4.0D+00*Y2
      RETURN
      END
C
C***********************************************************************
C
C FUNCTION TO CALCULATE PARAEQUILIBRIUM CARBON DIFFUSION CONTROLLED
C GROWTH RATE OF PEARLITE
C
      DOUBLE PRECISION FUNCTION VD(XGA,XGC,XAG,SC,D,S)
      IMPLICIT NONE
      DOUBLE PRECISION A,XAG,XGA,XGC,SC,D,S,SA,SCEM,CCEM
C
      A=0.72D+00
      SA=S*7.0D+00/8.0D+00
      SCEM=S/8.0D+00
      CCEM=0.25D+00
      VD=D/A*S**2/SA/SCEM*(XGA-XGC)/
     &                (CCEM-XAG)/S*(1.0D+00-SC/S)
      RETURN
      END
C
C**************************************************************
C
      DOUBLE PRECISION FUNCTION ENERGY2(T,T10,T20)
      IMPLICIT NONE
      DOUBLE PRECISION T,T7,T10,T20,F
C
      T7=T-1.0D+02*T20
      IF (T7 .LT. 3.0D+02) GOTO 1
      IF (T7 .LT. 7.0D+02) GOTO 2
      IF (T7 .LT. 9.21D+02) GOTO 3
      IF (T7 .LT. 1.0D+03) GOTO 5
      F=(3.381D+02-3.31D+00*(T7-1.0D+03)+9.83D-03*(T7-9.9999999D+02
     &)**1.96D+00-7.11D+00*DSIN(0.034*(T7-1.0D+03)))/(-4.187D+00)
      GOTO 4
1     F=1.38D+00*T7-1.499D+03
      GOTO 4
2     F=1.65786D+00*T7-1.581D+03
      GOTO 4
3     F=1.30089D+00*T7-1.331D+03
      GOTO 4
5     F=-1.0D+00*(1.20D+02-(T7-9.4D+02)*(0.73333333333D+00))
4     ENERGY2=(1.41D+02*T10 + F)*4.187D+00
      RETURN
      END
C
C
C***********************************************************************
C
      DOUBLE PRECISION FUNCTION X2AC0(T,X1,A,E11,E12)
      IMPLICIT NONE
	DOUBLE PRECISION A,E11,E12,T,X1
C
      X2AC0=(DLOG(A)-DLOG(X1)-E11*X1)/E12
C
      RETURN
      END
C
C***********************************************************************
C FUNCTION GIVING THE CARBON DIFFUSIVITY IN AUSTENITE
C
      DOUBLE PRECISION FUNCTION DIFF(T,X1,X2,W)
      IMPLICIT NONE
      DOUBLE PRECISION D(300),CARB(300)
	DOUBLE PRECISION ANS,XINCR,A,B,YA,YB,R,HH,KK,T
     &, A5,DASH,X,X1,X2,W,ACTIV,CG,DCG,DACTIV,THETA
     &, SIGMA,PSI
	INTEGER I,II3,IDIM,IERROR,IJMP,J,JMP,LAST,NA,Z
C
C HH=PLANCK CONST.J/S, KK=BOLTZMAN CONST. J/K
      PARAMETER (Z=12,R=8.31432D+00)
	PARAMETER (HH=6.6262D-34,KK=1.38062D-23)
      A5=1.0D+00
C
C     DIFF=DIFFUSIVITY OF CARBON IN AUSTENITE, CM**/SEC
C     Z=COORDINATION OF INTERSTIAL SITE
C     PSI=COMPOSITION DEPENDENCE OF DIFFUSION COEFFICIENT
C     THETA=NO. C ATOMS/ NO. FE ATOMS
C     ACTIV=ACTIVITY OF CARBON IN AUSTENITE
C     R=GAS CONSTANT
C     X=MOLE FRACTION OF CARBON
C     T=ABSOLUTE TEMPERATURE
C     SIGMA=SITE EXCLUSION PROBABLITY
C     W=CARBON CARBON INTERACTION ENERGY IN AUSTENITE
C
      DASH=(KK*T/HH)*DEXP(-(21230.0D+00/T))*DEXP(-31.84D+00)
      XINCR=(X1-X2)/300.0D+00
      DO 111 I=1,300
      CARB(I)=X2+(I-1)*XINCR
      X=CARB(I)
      THETA=X/(A5-X)
      ACTIV=CG(X,T,W,R)
      ACTIV=DEXP(ACTIV)
      DACTIV=DCG(X,T,W,R)
      DACTIV=DACTIV*ACTIV
      DACTIV=DACTIV*A5/((A5+THETA)**2)
      SIGMA=A5-DEXP((-(W))/(R*T))
      PSI=ACTIV*(A5+Z*((A5+THETA)/(A5-(A5+Z/2)*THETA+(Z/2)*(A5+Z/2)*
     &(A5-SIGMA)*THETA*THETA)))+(A5+THETA)*DACTIV
      D(I)=DASH*PSI
111   CONTINUE
      II3=0
C
C Calculation  the value of the integral over equidinstanly tabulated 
C function by means of Simpson's rule
C
      IDIM=300
      IERROR  =0
      JMP  =1
      IF (IDIM) 1,1,2
1     IERROR  =1000
      GOTO 4
2     ANS    =0.D+00
      IF (XINCR) 6,3,6
3     IERROR  =10
4     IF (IERROR+12345) 5,15,5
5     WRITE(*,100)'NO',IERROR
100   FORMAT (//' ***** DKO',I5,' RAIANSED ERROR INDICATOR TO ',I4,
     &     ' *****'///)
      GOTO 15
6     LAST =1+(IDIM-1)*JMP
      A    =0.D+00
      B    =0.D+00
      YA   =D(1)
      YB   =D(JMP+1)
      IF (IDIM-2) 15,13,7
7     J    =1
      NA   =IDIM/2
      IF (NA+NA-IDIM) 9,8,8
8     J    =J+JMP
      ANS    =ANS+1.125D+00*YA+2.375D+00*YB-0.625D+00*D(2*JMP+1)
     &     +0.125D+00*D(3*JMP+1)
9     I    =J
10    IF (I-LAST+JMP+JMP) 11,11,12
11    A    =A+D(I)
      IJMP =I+JMP
      B    =B+D(IJMP)
      I    =IJMP+JMP
      GOTO 10
12    ANS    =ANS+A+A+4.D+00*B-D(J)+D(LAST)
      GOTO 14
13    ANS    =(YA+YB)*1.5D+00
14    ANS    =ANS*XINCR/3.D+00
15    DIFF=ANS/(X1-X2)
      RETURN
      END
C
C
C
C**********************************************************************
C
C
C  Error Function
C
      DOUBLE PRECISION FUNCTION MAP_UTIL_DAERF(Y)
      IMPLICIT NONE            
      INTEGER           ISW,I
      DOUBLE PRECISION  P(5),Q(3),P1(8),Q1(7),P2(5),Q2(4)
      DOUBLE PRECISION  Y,XMIN,XLARGE,SSQPI,X,RES,XNUM,XDEN,XSQ,XI
C
C     COEFFICIENTS FOR 0.0 .LE. Y .LT. .477
C
      DATA               P(1)/-.44422647396874/,
     1                   P(2)/10.731707253648/,
     2                   P(3)/15.915606197771/,
     3                   P(4)/374.81624081284/,
     4                   P(5)/2.5612422994823D-02/
      DATA               Q(1)/17.903143558843/,
     1                   Q(2)/124.82892031581/,
     2                   Q(3)/332.17224470532/
C
C             COEFFICIENTS FOR .477 .LE. Y .LE. 4.0
C
      DATA               P1(1)/7.2117582508831/,
     1                   P1(2)/43.162227222057/,
     2                   P1(3)/152.98928504694/,
     3                   P1(4)/339.32081673434/,
     4                   P1(5)/451.91895371187/,
     5                   P1(6)/300.45926102016/,
     6                   P1(7)/-1.3686485738272D-07/,
     7                   P1(8)/.56419551747897/
      DATA               Q1(1)/77.000152935229/,
     1                   Q1(2)/277.58544474399/,
     2                   Q1(3)/638.98026446563/,
     3                   Q1(4)/931.35409485061/,
     4                   Q1(5)/790.95092532790/,
     5                   Q1(6)/300.45926095698/,
     6                   Q1(7)/12.782727319629/
C
C           COEFFICIENTS FOR 4.0 .LT. Y
C
      DATA               P2(1)/-.22695659353969/,
     1                   P2(2)/-4.9473091062325D-02/,
     2                   P2(3)/-2.9961070770354D-03/,
     3                   P2(4)/-2.2319245973418D-02/,
     4                   P2(5)/-2.7866130860965D-01/
      DATA               Q2(1)/1.0516751070679/,
     1                   Q2(2)/.19130892610783/,
     2                   Q2(3)/1.0620923052847D-02/,
     3                   Q2(4)/1.9873320181714/
C
C         CONSTANTS
C
      DATA               XMIN/1.0D-8/,XLARGE/5.6875D0/
      DATA               SSQPI/.56418958354776/
C
C       FIRST EXECUTABLE STATEMENT
C
      X   = Y
      ISW = 1
      IF (X.GE.0.0D0) GO TO 5
      ISW = -1
      X   = -X
C
5     CONTINUE
      IF (X.LT..477D0) GO TO 10
      IF (X.LE.4.0D0) GO TO 25
      IF (X.LT.XLARGE) GO TO 35
      RES = 1.D0
      GO TO 50
C                                  ABS(Y) .LT. .477, EVALUATE
C                                  APPROXIMATION FOR ERF
10    IF (X.LT.XMIN) GO TO 20
      XSQ = X*X
      XNUM = P(5)
      DO 15 I=1,4
         XNUM = XNUM*XSQ+P(I)
15    CONTINUE
C
      XDEN = ((Q(1)+XSQ)*XSQ+Q(2))*XSQ+Q(3)
      RES = X*XNUM/XDEN
      GO TO 50
20    RES = X*P(4)/Q(3)
      GO TO 50
C                                  .477 .LE. ABS(Y) .LE. 4.0
C                                  EVALUATE APPROXIMATION FOR ERF
25    XSQ = X*X
      XNUM = P1(7)*X+P1(8)
      XDEN = X+Q1(7)
C
      DO 30 I=1,6
         XNUM = XNUM*X+P1(I)
         XDEN = XDEN*X+Q1(I)
30    CONTINUE
C
      RES = XNUM/XDEN
      GO TO 45
C
C   4.0 .LT. ABS(Y), EVALUATE
C   APPROXIMATION FOR ERF
C
35    XSQ = X*X
      XI = 1.0D0/XSQ
      XNUM = P2(4)*XI+P2(5)
      XDEN = XI+Q2(4)
C
      DO 40 I=1,3
         XNUM = XNUM*XI+P2(I)
         XDEN = XDEN*XI+Q2(I)
40    CONTINUE
C
      RES = (SSQPI+XI*XNUM/XDEN)/X
45    RES = RES*DEXP(-XSQ)
      RES = 1.0D0-RES
50    IF (ISW.EQ.-1) RES = -RES
      MAP_UTIL_DAERF = RES
C
      RETURN
      END
C
C